#include "agreement.h"
#include "ui_agreement.h"
#include <QtGui/QPrinter>
#include <QtGui/QPrintDialog>
#include <QtGui/QTextDocument>
#include <QFile>
#include <QTextStream>
#include <QDebug>

#include <person.h>
#include <key.h>

//This creates the agreement screen and populates the agreement document
//It takes a pointer to a person object and REQUIRES ALL INFORMATION to be valid
//It also takes a pointer to a QList of pointers to key objects this REQUIRES THE KEY NAMES
//Finally it REQUIRES a QStringList where each item is a comma seperates list
// of rooms names, and item i in the QStringList applies to item i in the QList
Agreement::Agreement(person* p, QList<inventory*>* keys, QStringList* rooms, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Agreement)
{
    //creates all static gui elements
    ui->setupUi(this);
    connect(ui->printAgreement_but,SIGNAL(clicked()),this,SLOT(print()));


    QFile file("agreementText.txt");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
             return;

    QString text = "";

    //This reads the entire file into a QString
     QTextStream in(&file);
     while (!in.atEnd()) {
         text += in.readLine();
     }
     file.close();

     //update placeholders in HTML with own html
     QString keyString = "";
     for (int i = 0; i < keys->count(); i++)
     {
         key* k = (key*)keys->at(i);
         keyString += "<tr><td align=\"center\">"+*k->getKeyName()+"</td>";
         keyString += "<td><u>"+rooms->at(i)+"</u></td></tr>";
     }
     text.replace("%keys%",keyString);

     //All items will have the same due date, also a date of 0000-00-00
     //indicates no due date requested
     QDate zero(0,0,0);
     key *k = (key*)keys->at(0);
     QDate due = QDate::fromString(k->getDateDue(),Qt::ISODate);
     if (zero != due)
     {
         text.replace("%dueDate%","The following keys are due back on: "+due.toString());
     }
     else
     {
         text.replace("%dueDate%","");
     }
     QString address = QString(*p->getStreetAddress());
     text.replace("%strAddr%",address.replace("\n","<br/>"));
     text.replace("%Email%",*p->getEmail());
     text.replace("%CPhone%",*p->getCellPhone());
     text.replace("%OPhone%",*p->getWorkPhone());
     text.replace("%TNumber%",*p->getTNumber());
     text.replace("%Name%",*p->getFirstName()+" "+*p->getLastName());
     ui->textEdit->setHtml(text);
}

Agreement::~Agreement()
{
    delete ui;
}

//This uses the native printer configuration dialog on windows
//It allows for the printing of the richtext that was constructed in the constructor
void Agreement::print()
{
    QPrinter print;
    QPrintDialog options(&print);

    if (options.exec() == QDialog::Rejected)
    {
        return;
    }
    QTextDocument doc;
    doc.setHtml(ui->textEdit->toHtml());
    doc.print(&print);


}
