#include "checkin.h"
#include "ui_checkin.h"
#include "database.h"

#include "key.h"
#include "category_item.h"

extern void setErrorLabel(QLabel*,QString);

//Creates the screen enabling the user to select items for checkin
Checkin::Checkin(QList<inventory *>* inv, QWidget *parent):
    QDialog(parent),
    ui(new Ui::Checkin)
{
    //setup static gui elements
    ui->setupUi(this);

    //start of dynamic layout
    this->lay = new QGridLayout;
    //string required to convert integers to strings
    QString convert;
    for( int i = 0; i < inv->count(); i++)
    {
        QCheckBox* box;
        QLabel* table;
        inventory* item = inv->at(i);
        //display different if key or other
        //we also store the table name in a hidden label for later
        if (*item->getTable() == "pkeys")
        {
            box = new QCheckBox(*((key*)item)->getKeyName());
            table = new QLabel("pkeys");
        }
        else
        {
            box = new QCheckBox(*((category_item*)item)->getDescriptor());
            table = new QLabel(*item->getTable());
        }
        checks.append(box);
        tables.append(table);
        //also we hide the transaction id in a label
        convert = convert.setNum(item->getCheckOutTransaction());
        QLabel* transID = new QLabel(convert);
        transactions.append(transID);
        //set the combo box so that it only allows checkins in the range
        //of numbers that the borrower has checked-out
        QComboBox* amount;
        amount = new QComboBox;
        for (int j = 1; j <= item->getNumberCheckedOut(); j++)
        {
           convert = convert.setNum(j);
           amount->addItem(convert);
        }
        amount->setCurrentIndex(amount->count()-1);
        quantities.append(amount);

        table->hide();
        transID->hide();
        lay->addWidget(box,i,0);
        lay->addWidget(amount,i,1);
        lay->addWidget(table,i,2);
        lay->addWidget(transID,i,3);
        //rinse, lather, repeat
    }
    setLayout(lay);
}

//This function will attempt to checkin the amounts selected of the items requested
void Checkin::accept()
{
    //to hold the selected inventory
    QList<inventory*> inv;
    for (int i = 0; i < checks.count(); i++)
    {
        QCheckBox* box = checks.at(i);
        if (box->isChecked())
        {
            inventory* item;
            //again, keys are special
            if (tables.at(i)->text() == "pkeys")
            {//keys
                key* k = new key;
                k->setTable("pkeys");
                k->setCheckOutTransaction(transactions.at(i)->text().toInt());
                k->setKeyName(box->text());
                //here current number is the amount to check-in
                k->setCurrentNumber(quantities.at(i)->currentText().toInt());
                k->setNumberCheckedOut(quantities.at(i)->count());
                item = k;
            }
            else
            {//other invnentory
                category_item* ci = new category_item;
                ci->setTable(tables.at(i)->text());
                ci->setDescriptor(box->text());
                ci->setCheckOutTransaction(transactions.at(i)->text().toInt());
                //here current number is the amount to check-in
                ci->setCurrentNumber(quantities.at(i)->currentText().toInt());
                ci->setNumberCheckedOut(quantities.at(i)->count());

                item = ci;
            }
            inv.append(item);
        }
    }

    if (inv.count() == 0)
    {
        setErrorLabel(ui->CheckinLabel,"You must select at least one item to check-in.");
        return;
    }
    Database *db = NULL;
    try
    {
        db = new Database("checkin");
        //all or nothing
        db->transaction();

        for (int i = 0; i < inv.count(); i++)
        {

            if (!db->CheckInInventory(inv.at(i)))
            {
                setErrorLabel(ui->CheckinLabel,"That inventory could not be checked-in at this time.");
                //nothing
                db->rollback();
                return;
            }
        }
        //all
        if (db->commit())
        {
            qDeleteAll(inv.begin(),inv.end());
            delete db;
            QDialog::accept();
        }
        else
        {
            setErrorLabel(ui->CheckinLabel,"Those items could not be checked in at this time.");
            db->rollback();
            qDeleteAll(inv.begin(),inv.end());
            delete db;
        }
    }
    catch (int i){
        qDebug() << "error";
        if (i == 1){
            setErrorLabel(ui->CheckinLabel,"The database connection failed.<br>Please try again later.");
        }
        else if (i == 2){
            setErrorLabel(ui->CheckinLabel,"The database connection was lost.<br>Please try again later.");
        }
        else if (i == 8)
        {
            setErrorLabel(ui->CheckinLabel,"One or more of the categories you know<br>of have been removed.");
        }
        else {
            setErrorLabel(ui->CheckinLabel,"There was an unknown error.");
        }
        qDeleteAll(inv.begin(),inv.end());
        if (db != NULL)
        {
            db->rollback();
            delete db;
        }
    }

}


Checkin::~Checkin()
{
    delete ui;
    qDeleteAll(checks.begin(),checks.end());
    qDeleteAll(tables.begin(),tables.end());
    qDeleteAll(transactions.begin(),transactions.end());
    qDeleteAll(quantities.begin(),quantities.end());
    delete lay;
}
