#include "checkout.h"
#include "ui_checkout.h"

#include "key.h"
#include "category_item.h"

//Creates the screen enabling the user to select amounts to check-out as well as
//whether a due date is requested
//It takes a pointer to a QList pointers to inventory items, each of which
// is REQUIRED to have DESCRIPTOR/KEY NAME and NUMBER AVAILABLE
//It also takes a pointer to a KeyWindow object, this is to set instance variables
//required by the keyWindows processing
Checkout::Checkout(QList<inventory*>* inv, KeyWindow* k, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Checkout)
{
    //setup static gui elements
    ui->setupUi(this);
    this->k = k;
    //start of dynamic gui
    topLayout = new QVBoxLayout;
    lay = new QGridLayout;
    names = new QList<QLabel*>;
    amounts = new QList<QComboBox*>;

    amountString = new QLabel("Please select inventory items and amounts to check-out.");
    topLayout->addWidget(amountString);

    for (int i = 0; i < inv->count(); i++)
    {
        QLabel* name = new QLabel;

        inventory* item = inv->at(i);
        //treat keys special
        if (*inv->at(i)->getTable() == "pkeys")
        {
            name->setText(*((key*)item)->getKeyName());
            lay->addWidget(name,i,0);
        }
        else
        {
            name->setText(*((category_item*)item)->getDescriptor());
            lay->addWidget(name,i,0);
        }

        names->append(name);
        //set the combo box to allow the user to only select an amount in
        //the valid range
        QComboBox* amount = new QComboBox;
        QStringList numbers;
        QString convert;
        for (int j = 1; j <= inv->at(i)->getAmountAvailable(); j++)
        {
            convert = convert.setNum(j);
            numbers.append(convert);
        }
        amount->addItems(numbers);
        amounts->append(amount);

        lay->addWidget(amount,i,1);
    }

    topLayout->addLayout(lay);
    //the check box
    date = new QCheckBox("Due Date Requested?");
    topLayout->addWidget(date);
    topLayout->addWidget(ui->buttonBox);


    setLayout(topLayout);


}

//This calls functions on the KeyWindow pointer to continue the check-out process
void Checkout::accept()
{
    QList<int> quantities;
    for (int i = 0; i < amounts->count(); i++)
    {
        QComboBox* box = this->amounts->at(i);
        quantities.append(box->currentText().toInt());
    }
    //give the list of amounts and the request back to the KeyWindow
    k->setCheckoutQuantity(quantities);
    k->setDueDateRequest(this->date->isChecked());

    QDialog::accept();
}

Checkout::~Checkout()
{
    qDeleteAll(amounts->begin(),amounts->end());
    delete amounts;
    delete amountString;
    qDeleteAll(names->begin(),names->end());
    delete names;
    delete date;
    delete lay;
    delete topLayout;
    delete ui;


}
