#include "database.h"
#include "duedate.h"

//This function takes a QDate indicating the selected month and a StringList
// containing all of the invenotry categories in the sysem EXCEPT keys.
// It will return a pointer to a QList with pointers to dueDate objects.
// BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//
//This function could throw an integer on connection failure, or if a category does not exist
QList<dueDate*>* Database::getDueWithinMonth(QDate month, QStringList categories)
{
    QSqlQuery query(*dbs);
    QString queryString = "select distinct DateDue,TransactionID,TNumber from ";
    QString whereString = " where DateDue LIKE ?";

    QString searchDate = month.toString(Qt::ISODate);
    //replace the day numbers with one character wildcards to get all in a month and year
    searchDate.replace(8,2,"__");

    QList<dueDate*>* dates = new QList<dueDate*>;  

    //loop runs for all categories
    for (int i = 0; i < categories.count(); i++)
    {
        QString searchString;
        searchString = queryString+"p_"+categories.at(i);
        query.prepare(searchString+whereString);
        query.bindValue(0,searchDate);
        if (!query.exec())
        {
            throw getErrorNumber(query.lastError());
        }
        while (query.next())
        {
            dueDate* date = new dueDate;
            date->setTable(categories.at(i));
            date->setDueDate(query.value(0).toString());
            date->setTransactionID(query.value(1).toInt());
            date->setTNumber(query.value(2).toString());
            dates->append(date);
        }
    }
    //this is a copy of what is inside the loop, except for the table names used
    //MORAL: keys are special
    QString searchString;
    searchString = queryString+"p_k";
    query.prepare(searchString+whereString);
    query.bindValue(0,searchDate);
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    while (query.next())
    {
        dueDate* date = new dueDate;
        date->setTable("pkeys");
        date->setDueDate(query.value(0).toString());
        date->setTransactionID(query.value(1).toInt());
        date->setTNumber(query.value(2).toString());
        dates->append(date);
    }
    return dates;

}
