#include "database.h"
#include "key.h"
#include "room.h"

//This takes a pointer to a key object and REQUIRES EITHER TAG NUMBER or KEY NAME be valid
//It also takes a pointer to a QList with pointers to room objects and REQUIRES
// A VALID ROOMID FOR EACH
// This function attempts to find all keys in the system that matches the given
//tag number and/or key name with the mysql LIKE '%tag number%' or LIKE '%key name%'
// and can open all rooms that are passed in.
//The function returns a pointer to a QList with pointers to key objects
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any keys in the list are ordered alphabetically by key name.
//All keys returned will have valid tag numbers, key names, totals, and available amounts
//
//This can throw an int on database connection error
QList<key*>* Database::getKeys(key* k, QList<room*>* rooms)
{
    QList<key*>* keys = new QList<key*>;
    QStringList params;
    QSqlQuery query(*dbs);
    QString queryString = "select distinct TagNum,KeyName,Total,InInv from pkeys";

    QString num;
    bool first = true;
    if (k->getTagNum() != 0)
    {
        if (!first)
        {//technically this cannot happen
            queryString += " AND";
        }
        else
        {
            queryString += " where ";
            first = !first;
        }
        queryString += " TagNum LIKE ?";

        num = num.setNum(k->getTagNum());
        params.append("%"+num+"%");
    }
    if (*k->getKeyName() != "")
    {
        if (!first)
        {
            queryString += " AND";
        }
        else
        {
            queryString += " where ";
            first = !first;
        }
        queryString += " KeyName LIKE ?";
        params.append("%"+*k->getKeyName()+"%");

    }
    if (rooms->count() != 0)
    {
        queryString += " AND TagNum in (select TagNum from k_r where ";
        bool first2 = true;
        for (int i = 0; i < rooms->count(); i++)
        {
            if (first2)
            {
                first2 = false;
            }
            else
            {
                queryString += "AND ";
            }

            queryString += "RoomId = ? ";
            num = num.setNum(rooms->at(i)->getRoomID());
            params.append(num);
        }
    }
    queryString += " ORDER BY KeyName";
    qDebug() << queryString;
    query.prepare(queryString);

    //bind all of the parameters that I have collected along the way
    for (int i = 0; i < params.count(); i++){
        query.bindValue(i,params.at(i));
    }
    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    while (query.next())
    {
        key* ci = new key;
        ci->setTagNum(query.value(0).toInt());
        ci->setKeyName(query.value(1).toString());
        ci->setTotal(query.value(2).toInt());
        ci->setAmountAvalable(query.value(3).toInt());
        ci->setTable("pkeys");
        keys->append(ci);
    }
    return keys;
}

// This function attempts to find all keys in the system
//The function returns a pointer to a QList with pointers to key objects
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any keys in the list are ordered alphabetically by key name.
//All keys returned will have valid tag numbers, key names, totals, and available amounts
//
//This can throw an int on database connection error
QList<key*>* Database::getAllKeys()
{
    QList<key*>* keys = new QList<key*>;
    QSqlQuery query(*dbs);
    QString queryString = "select distinct TagNum,KeyName,Total,InInv from pkeys ORDER BY KeyName";
    query.prepare(queryString);

    if (!query.exec())
    {
        throw getErrorNumber(query.lastError());
    }
    while (query.next())
    {
        key* ci = new key;
        ci->setTagNum(query.value(0).toInt());
        ci->setKeyName(query.value(1).toString());
        ci->setTotal(query.value(2).toInt());
        ci->setAmountAvalable(query.value(3).toInt());
        ci->setTable("pkeys");
        keys->append(ci);
    }
    return keys;
}

//This takes a pointer to a key object and REQUIRES a TAG NUMBER be supplied
//This will attempt to retrieve all rooms opened by a key in the system.
//This returns a pointer to a QList with pointers to room objects.
//BOTH QLIST and ALL OBJECTS WITHIN MUST BE DELETED BY CALLER
//The list can be empty, but any rooms in the list are ordered alphabetically by group name.
//All rooms returned will have valid room names, and room id's.
//
//This can throw an int on database connection error
QList<room*>* Database::getKeyRooms(key* k)
{
    QSqlQuery query(*dbs);
    QString queryString = QString("select distinct RoomName,RoomID from rooms where ")+
                          QString("RoomID in (select distinct RoomID from k_r where TagNum = ?) ORDER BY RoomName");
    query.prepare(queryString);
    query.bindValue(0,k->getTagNum());
    if (!query.exec()){
        throw getErrorNumber(query.lastError());
    }
    QList<room*>* rooms = new QList<room*>;
    while (query.next()){
        room* get = new room;
        get->setRoomName(query.value(0).toString());
        get->setRoomID(query.value(1).toInt());
        rooms->append(get);
    }
    return rooms;
}
