#include "groupselection.h"
#include "ui_groupselection.h"
#include "database.h"

#include "group.h"
#include "person.h"

extern void setErrorLabel(QLabel*,QString);

//This sets up the gui for allowing the user to select groups to remove users from
//The pointer to a person MUST have a valid TNumber
//The pointers to groups MUST have valid Group names and Group IDS's
GroupSelection::GroupSelection(person* p, QList<group*>* groups, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::GroupSelection)
{
    //set up static gui
    ui->setupUi(this);
    this->p = p;
    //start dynamic gui
    this->lay = new QGridLayout;
    for( int i = 0; i < groups->count(); i++)
    {
        group* g = groups->at(i);
        //display group name with check box
        QCheckBox* box = new QCheckBox(*g->getGroupName());
        checks.append(box);
        QString convert;
        convert = convert.setNum(g->getGroupID());
        QLabel* id = new QLabel(convert);
        //hide group ID
        id->hide();
        labels.append(id);
        lay->addWidget(box,i,0);
        lay->addWidget(id,i,1);
    }
    setLayout(lay);
}

//Attempt to remove the user from the groups given
void GroupSelection::accept()
{
    // if the group is checked add it to a new list of groups
    QList<group*> groups;
    for (int i = 0; i < checks.count(); i++)
    {
        QCheckBox* box = checks.at(i);
        if (box->isChecked())
        {
            group* g = new group(labels.at(i)->text().toInt(),box->text());
            groups.append(g);
        }
    }
    Database *db = NULL;
    try
    {
        db = new Database("removeUsers");
        db->transaction();
        for( int i = 0; i < groups.count(); i++)
        {
            db->removePersonFromGroup(groups.at(i),p);
        }
        if (db->commit())
        {
            qDeleteAll(groups.begin(),groups.end());
            delete db;
            QDialog::accept();
        }
        else
        {
            setErrorLabel(ui->GroupSelectionLabel,"The person could not be removed<br>from those groups.");
            qDeleteAll(groups.begin(),groups.end());
            db->rollback();
            delete db;
        }
    }
    catch (int i)
    {
        qDebug() << "error";
        if (i == 1)
        {
            setErrorLabel(ui->GroupSelectionLabel,"The database connection failed.<br>Please try again later.");
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupSelectionLabel,"The database connection was lost.<br>Please try again later.");
        }
        else
        {
            setErrorLabel(ui->GroupSelectionLabel,"There was an unknown error.");
        }
        qDeleteAll(groups.begin(),groups.end());
        if (db != NULL)
        {
            db->rollback();
            delete db;
        }
    }


}

GroupSelection::~GroupSelection()
{
    delete ui;
    qDeleteAll(checks.begin(),checks.end());
    qDeleteAll(labels.begin(),labels.end());
    delete lay;
}
