#ifndef INEVENTORY_H
#define INEVENTORY_H

#include <QString>
#include <QtCore/QDate>

//This class is used to hold information about either keys or other, category, items
//in the system, as well as holding information about a specific item being checked out
//This is the parent class of category_item and key
class inventory
{

public:
    inventory();
    inventory(const QString&,int = 0, int = 0);
    ~inventory();
    const QString* getTable();
    int getCurrentNumber();
    int getTotal();
    const QString getDateOut();
    const QString getDateDue();
    int getCheckOutTransaction();
    int getAmountAvailable();
    int getNumberCheckedOut();

    void setTable(const QString&);
    void setCurrentNumber(int);
    void setTotal(int);
    void setDateOut(QString);
    void setDateDue(QString);
    void setCheckOutTransaction(int);
    void setAmountAvalable(int);
    void setNumberCheckedOut(int);

private:
    QString* table;
    QDate dateOut;
    QDate dateDue;
    int checkOutTransaction;
    int inInv;
    int numOut;
    //Note following int, currNum is used for many things, such as the number
    //requested to check-out or check-in, or the initial inventory
    int currNum;
    int total;
};

//This returns a pointer to a QString with the table name of this inventory
inline const QString* inventory::getTable()
{
    return this->table;
}

//This returns the current number, see note in class definition
inline int inventory::getCurrentNumber()
{
    return this->currNum;
}

//This returns the total number of that item in the inventory
inline int inventory::getTotal()
{
    return this->total;
}

//This returns a QString with the date the item was checked out in ISO format
inline const QString inventory::getDateOut()
{
    return this->dateOut.toString(Qt::ISODate);
}

//This returns a QString with the date the item is due back in ISO format
inline const QString inventory::getDateDue()
{
    return this->dateDue.toString(Qt::ISODate);
}

//This returns the check-out transaction number
inline int inventory::getCheckOutTransaction()
{
    return this->checkOutTransaction;
}

//This returns the about available to check-out
inline int inventory::getAmountAvailable()
{
    return this->inInv;
}

//This returns the number of the items that are checked out with the transaction id
inline int inventory::getNumberCheckedOut()
{
    return this->numOut;
}

//This deletes the old table string, and resets it with a copy of the string passed in
inline void inventory::setTable(const QString& table)
{
    delete this->table;
    this->table = new QString(table);
}

//This sets the current number, see note above
inline void inventory::setCurrentNumber(int curr)
{
    this->currNum = curr;
}

//This sets the total about of the item in the inventory
inline void inventory::setTotal(int total)
{
    this->total = total;
}

//This sets the date out by reading the date from a string in ISO format
inline void inventory::setDateOut(QString date)
{
    this->dateOut = QDate::fromString(date,Qt::ISODate);
}

//This sets the due date by reading the date from a string in ISO format
inline void inventory::setDateDue(QString date)
{
    this->dateDue = QDate::fromString(date,Qt::ISODate);
}

//This sets the transaction id for the check-out
inline void inventory::setCheckOutTransaction(int trans)
{
    this->checkOutTransaction = trans;
}

//This sets the amount of this item still available for check-out
inline void inventory::setAmountAvalable(int amt)
{
    this->inInv = amt;
}

//This sets the number of this item checked-out with the given transaction id
inline void inventory::setNumberCheckedOut(int numOut)
{
    this->numOut = numOut;
}

#endif // INEVENTORY_H
