#include <QtGui>
#include "keywindow.h"
#include "ui_keywindow.h"
#include "dataconfig.h"
#include "user.h"
#include "database.h"

#include "person.h"
#include "group.h"
#include "inventory.h"


extern void setErrorLabel(QLabel*, QString);

//This fucntion creates the gui for the main screen, and sets all of the instance
//variables needed for the class, it calls funstions in the other KeyWindow implementation
//files to connect the buttons to their actions
KeyWindow::KeyWindow(QWidget *parent) :QMainWindow(parent),ui(new Ui::KeyWindow)
{

    ui->setupUi(this);
    try{
        this->db = new Database("inventory");
    }
    catch (int i){
        if (i == 1){
            setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else {
            setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
        }
    }

    per = NULL;
    grp = NULL;
    inv = NULL;
    dueDateRequest = false;
    paths = QByteArray(2,'\0');

    this->month = 1;
    this->year = 2000;

    this->quantityVal = new QIntValidator(-300,300,this);
    this->tagVal = new QIntValidator(0,1001,this);
    QRegExp tnumber("T?\\d{8}");
    this->tnumVal = new QRegExpValidator(tnumber,0);
    QRegExp phone("\\d{3}-\\d{3}-\\d{4}");
    this->phoneVal = new QRegExpValidator(phone,0);
    this->selPersonInfo = new person;
    this->selPersonGroups = new QList<group*>;
    this->selPersonInv = new QList<inventory*>;
    this->currentMonth = new QList<dueDate*>;

    ui->listWidgetPeople->setSelectionMode(QAbstractItemView::ExtendedSelection);
    ui->listWidgetKeys->setSelectionMode(QAbstractItemView::ExtendedSelection);
    ui->listWidgetCategories->setSelectionMode(QAbstractItemView::ExtendedSelection);


    connect(ui->DatabaseConfigA, SIGNAL(triggered()), this, SLOT(DatabaseConfiguration()));
    connect(ui->userManA, SIGNAL(triggered()), this, SLOT(UserMan()));
    connect(ui->tabWidget,SIGNAL(currentChanged(int)),this,SLOT(tabChange(int)));

    updateCategories();


    peopleConnections();
    roomConnections();
    groupConnections();
    otherInvConnections();
    keyConnections();
    calendarConnections();
}

//Fucntion to do bookkeeping when a tab is changed
void KeyWindow::tabChange(int tab){
    if (tab == KeyWindow::PEOPLE){
        //This all deals with the checkout and add to group procedure
        if (KeyWindow::NONE != isPathActive() &&
            ( grp != NULL || inv != NULL )&& per == NULL)
        {
            qDebug() << (int)isPathActive();
            ui->continuePerson_but->setEnabled(true);
            ui->cancelPerson_but->setEnabled(true);
            if (this->dueDateRequest == false)
            {
                ui->continuePerson_but->setText("Finish");
            }
        }
        else {
            ui->continuePerson_but->setEnabled(false);
            ui->continuePerson_but->setText("Continue");
            ui->cancelPerson_but->setEnabled(false);
        }
    }
    if (tab == KeyWindow::ROOMS){
        //show the lastest list of rooms
        listRooms();
    }
    if (tab == KeyWindow::GROUPS){
        //show the latest list of groups
        listGroups();
        //deals with adding groups to rooms
        if (KeyWindow::NONE != isPathActive()){
            ui->addPeopletoGroup_but->setEnabled(false);
        }
        else{
            ui->addPeopletoGroup_but->setEnabled(true);
        }
    }
    if (tab == KeyWindow::INV){
        //the following line caused unacceptable delay as of yet
        //updateCategories();
    }
    if (tab == KeyWindow::CALENDAR){
        //Dealing with checking out inventory
        if (KeyWindow::NONE != isPathActive() && per != NULL && dueDateRequest){
            ui->continueCalendar_but->setEnabled(true);
            ui->cancelCalendar_but->setEnabled(true);
        }
        else {
            ui->continueCalendar_but->setEnabled(false);
            ui->cancelCalendar_but->setEnabled(false);
        }
    }
}

//allows the check-out window to tell us how many of each item
void KeyWindow::setCheckoutQuantity(QList<int> quantities)
{
    for (int i = 0; i < this->inv->count(); i++)
    {
        this->inv->at(i)->setCurrentNumber(quantities.at(i));
    }
}

//allows the check-out window to tell us if a due date is requested
void KeyWindow::setDueDateRequest(bool request)
{
    this->dueDateRequest = request;
}

//This reads the list of other categories and places them into the
//category selector on the categories tab
void KeyWindow::updateCategories(){
    try
    {
        if (this->db == NULL){
            this->db = new Database("inventory");
        }
        this->categories = this->db->getCategory();
        ui->selectionListCategories->clear();
        if (categories.count() == 0){
            ui->selectionListCategories->setEnabled(false);
            ui->selectionListCategories->addItem("No Categories");
            return;
        }
        else {
            ui->selectionListCategories->setEnabled(true);
            ui->selectionListCategories->addItem("None Selected");
            for (int i = 0; i < categories.count(); i++){
                ui->selectionListCategories->addItem(categories.at(i));
            }
            return;
        }
    }
    catch (int i){
        if (i == 1){
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2){
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
        return;
    }

}


//Allows for the database configuration to be changed for next login
void KeyWindow::DatabaseConfiguration(){
    DataConfig d;
    if (d.exec() == QDialog::Accepted){
        ui->PersonErrorLabel->setText("Your database configuration was successfully saved.");
        ui->RoomErrorLabel->setText("Your database configuration was successfully saved.");
        ui->KeysErrorLabel->setText("Your database configuration was successfully saved.");
        ui->GroupErrorLabel->setText("Your database configuration was successfully saved.");
        ui->CategoryErrorLabel->setText("Your database configuration was successfully saved.");
        ui->CalendarErrorLabel->setText("Your database configuration was successfully saved.");

    }
}

//Display the user management screen
void KeyWindow::UserMan(){
    User u;
    u.exec();
}

//Strip white space off of the front and back of room names
//while processing them
QStringList KeyWindow::getTrimmedRooms(QString roomString){
    QStringList old = roomString.split(',');
    QStringList rooms;
    for(int i = 0; i < old.count(); i++){
        QString add = old.at(i).trimmed();
        rooms.append(add);
    }
    return rooms;
}

//Determines whether a cross tab process is in process
KeyWindow::PATH KeyWindow::isPathActive() {
    if (paths.at(0) == '\1'){
        return KeyWindow::ADD_TO_GROUP;
    }
    else if (paths.at(1) == '\1'){
        return KeyWindow::CHECK_OUT;
    }
    else {
        return KeyWindow::NONE;
    }
}

//Attempts to begin a cross tab process
//returns a boolean indicating sucess
bool KeyWindow::setPath(KeyWindow::PATH path) {

    KeyWindow::PATH currPath = isPathActive();
    if (currPath != KeyWindow::NONE){
        return false;
    }
    else {
        if (path == KeyWindow::ADD_TO_GROUP){
            paths.replace(0,1,"\1",1);
            return true;
        }
        else if (path == KeyWindow::CHECK_OUT){
            paths.replace(1,1,"\1",1);
            return true;
        }
        return false;

    }
}

//reset the path byte array
void KeyWindow::clearPaths(){
    paths.replace('\1','\0');
}


//clear all indication of old cross tab process
void KeyWindow::pathEnd(){
    clearPaths();
    delete grp;
    if (this->inv != NULL && this->inv->count() > 0)
    {
        qDeleteAll(inv->begin(),inv->end());
    }
    delete inv;
    delete per;
    grp = NULL;
    inv = NULL;
    per = NULL;
    dueDateRequest = false;
    ui->continuePerson_but->setEnabled(false);
    ui->cancelPerson_but->setEnabled(false);
    ui->continueCalendar_but->setEnabled(false);
    ui->cancelCalendar_but->setEnabled(false);
}

KeyWindow::~KeyWindow()
{
    delete ui;
    delete this->db;
    delete per;
    delete grp;
    delete inv;
    delete quantityVal;
    delete tagVal;
    delete tnumVal;
    delete phoneVal;
}
