#ifndef KEYWINDOW_H
#define KEYWINDOW_H

#include <QMainWindow>
#include <QListWidgetItem>
#include <database.h>

class QAction;
class QMenu;

namespace Ui {
    class KeyWindow;
}

//This class has all of the functions and functionality of the big with with
//the tabs, therefore its definitions have been split up into several files
class KeyWindow : public QMainWindow
{
    Q_OBJECT

public:

    explicit KeyWindow(QWidget *parent = 0);
    ~KeyWindow();

    //These need to be public so the check out window can set the quantities
    //to checkout and the due date request
    void setCheckoutQuantity(QList<int>);
    void setDueDateRequest(bool);


private:
    Ui::KeyWindow *ui;

    //This enum is used to indicate which of my process that cross tabs is currently
    //going on
    enum PATH {NONE = 0, ADD_TO_GROUP = 1, CHECK_OUT = 2};
    //This enum holds the indices for my tabs, it makes it easier to switch tabs
    enum TABS {PEOPLE = 0, ROOMS = 1, KEYS = 2, GROUPS = 3, INV = 4, CALENDAR = 5};

    //Used for adding to groups, and checking-out inventory
    person* per;
    group* grp;
    QList<inventory*>* inv;
    bool dueDateRequest;
    QByteArray paths;
    //

    //the list of other inventory categories
    QStringList categories;
    //The validators for the numeric inputs
    QValidator *quantityVal, *tagVal;
    //The Tnumber validator
    QValidator *tnumVal;
    //The phone number validator
    QValidator *phoneVal;

    //Used for showing the items due in a given month on the calendar
    QList<dueDate*>* currentMonth;
    int month;
    int year;
    //

    //The resident database connection
    Database *db;

//Utility functions, in keywindow.cpp
private slots:
    void DatabaseConfiguration();
    void tabChange(int);
    void UserMan();
    void updateCategories();
    QStringList getTrimmedRooms(QString);
    PATH isPathActive();
    bool setPath(KeyWindow::PATH);
    void clearPaths();
    void pathEnd();






////////////////////////////////////////
// $People, in keywindow_people.cpp
///////////////////////////////////////
private slots:
    void peopleConnections();
    void displayPersonInfo(QListWidgetItem*);
    void addPerson();
    void updatePerson();
    void deletePerson();
    void queryPeople();
    void personClear();
    void personPrepare(QListWidgetItem*);
    void resetCustom();
    void checkinInventory();
    void continue_path();
    void removePersonFromGroup();
    void addPersonToGroup_person();
    void cancelPath_person();
    void checkout_person();
    QString getPersonInfoString(person*, QList<group*>*, QList<inventory*>*);

private:
    QList<group*>* selPersonGroups;
    QList<inventory*>* selPersonInv;
    person* selPersonInfo;

///////////////////////////////////////////
// $End_People
///////////////////////////////////////////

//////////////////////////////////////////
// $Rooms, in keywindow_rooms.cpp
//////////////////////////////////////////
private slots:
    void roomConnections();
    void displayRoomInfo(QListWidgetItem*);
    void addRoom();
    void deleteRoom();
    void roomClear();
    void listRooms();
    void roomPrepare(QListWidgetItem *);
    void findPeopleWithRooms();
    QString getRoomInfoString(room*, QList<group *>*, QList<key *>*);


//////////////////////////////////////////
// $End_Rooms
//////////////////////////////////////////

//////////////////////////////////////////
// $Groups, in keywindow_groups.cpp
//////////////////////////////////////////
private slots:
    void groupConnections();
    void displayGroupInfo(QListWidgetItem*);
    void addGroup();
    void deleteGroup();
    void groupClear();
    void listGroups();
    void addPersonToGroup_group();
    void removeAllPeople();
    void groupPrepare(QListWidgetItem*);
    void modifyGroupRoomsList_add();
    void modifyGroupRoomsList_remove();
    void findPeopleInGroups();
    QString getGroupInfoString(group*,QList<room*>*,QList<person*>*);


//////////////////////////////////////////
// $End_Groups
//////////////////////////////////////////

//////////////////////////////////////////
// $Keys, in keywindow_keys.cpp
//////////////////////////////////////////
private slots:
    void keyConnections();
    void displayKeyInfo(QListWidgetItem*);
    void addKey();
    void changeKeyQuantity();
    void keyClear();
    void keyPrepare(QListWidgetItem*);
    void listKeys();
    void queryKeys();
    void checkout_keys();
    void findPeopleWithKey();
    void modifyKeyRoomsList_add();
    void modifyKeyRoomsList_remove();
    QString getKeyInfoString(key*,QList<room*>*);

//////////////////////////////////////////
// $End_Keys
//////////////////////////////////////////

//////////////////////////////////////////
// $Other_Inventory, in keywindow_otherinv.cpp
//////////////////////////////////////////

private slots:
    void otherInvConnections();
    void displayInventoryInfo(QListWidgetItem*);
    void addCategory();
    void removeCategory();
    void addItem();
    void changeItemQuantity();
    void itemClear(int);
    void itemPrepare(QListWidgetItem*);
    void queryItems();
    void checkout_otherInv();
    void findPeopleWithInventory();
    QString getOtherInvInfoString(category_item*);

//////////////////////////////////////////
// $End_Other_Inventory
//////////////////////////////////////////

//////////////////////////////////////////
// $Calendar, in keywindow_calendar.cpp
//////////////////////////////////////////

signals:
    void selectedDate(const QDate&);

private slots:
    void calendarConnections();
    void getDueInSelectedMonth(int,int);
    void getDueOnSelectedDay();
    void displayDueDate(QListWidgetItem*);
    void checkout_calendar();
    void cancelPath_calendar();
    void setCalendarDate();

//////////////////////////////////////////
// $End_Calendar
//////////////////////////////////////////

};

#endif // KEYWINDOW_H
