#include <QtGui>
#include "keywindow.h"
#include "ui_keywindow.h"
#include "database.h"
#include "agreement.h"
#include "noroom.h"

#include "duedate.h"
#include "key.h"
#include "category_item.h"
#include "room.h"
#include "person.h"


extern void setErrorLabel(QLabel*, QString);

//connect buttons and other signals to functions in the system
void KeyWindow::calendarConnections()
{
    connect( ui->calendarWidget, SIGNAL(clicked(QDate)), ui->dateEdit, SLOT(setDate(QDate)));
    connect( ui->calendarWidget, SIGNAL(currentPageChanged(int,int)), this, SLOT(getDueInSelectedMonth(int,int)));
    connect( ui->calendarWidget, SIGNAL(selectionChanged()), this, SLOT(getDueOnSelectedDay()));
    connect( ui->dateEdit, SIGNAL(editingFinished()), this, SLOT(setCalendarDate()));
    connect( this, SIGNAL(selectedDate(const QDate&)), ui->calendarWidget, SLOT(setSelectedDate(QDate)));
    connect( ui->continueCalendar_but, SIGNAL(clicked()),this,SLOT(checkout_calendar()));
    connect( ui->cancelCalendar_but, SIGNAL(clicked()), this, SLOT(cancelPath_calendar()));
    connect( ui->listWidgetCalendar, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(displayDueDate(QListWidgetItem*)));
}

//Takes two integers representing the year and month that the calendar is on now.
//If these have changed from the previous values, then the database is queried to
//find all days where something is due back , and display these on the calendar
void KeyWindow::getDueInSelectedMonth(int currYear, int currMonth)
{
    if (currYear != year || currMonth != month)
    {
        QBrush brush;
        QTextCharFormat cf;
        brush.setColor(Qt::white);
        cf.setBackground(brush);
        //first clear the backgroud of the dates in the old month
        for (int i = 0; i < this->currentMonth->count(); i++)
        {
            dueDate* date = currentMonth->at(i);
            QDate clear = QDate::fromString(date->getDueDate(),Qt::ISODate);
            ui->calendarWidget->setDateTextFormat(clear,cf);

        }
        qDeleteAll(currentMonth->begin(),currentMonth->end());
        delete currentMonth;

        //the day doesn't matter
        QDate selected(currYear,currMonth,1);

        try{
            if (this->db == NULL)
            {
               this->db = new Database("inventory");
               ui->CalendarErrorLabel->setText("Database connection re-established.");

            }
            this->currentMonth = this->db->getDueWithinMonth(selected,this->categories);
        }
        catch (int i)
        {
            if (i == 1)
            {
                setErrorLabel(ui->CalendarErrorLabel,"The database connection failed.<br>Please try again later.");
                delete this->db;
                this->db = NULL;
            }
            else if (i == 2)
            {
                setErrorLabel(ui->CalendarErrorLabel,"The database connection was lost.<br>Please try again later.");
                delete this->db;
                this->db = NULL;
            }
            else if (i == 6)
            {
                setErrorLabel(ui->CalendarErrorLabel,"At least one of the categories you think<br>exists has been removed from the sysetm.");
            }
            else
            {
                setErrorLabel(ui->CalendarErrorLabel,"There was an unknown error.");
            }
            year = currYear;
            month = currMonth;
            this->currentMonth = new QList<dueDate*>;
            return;
        }

        for (int i = 0; i < currentMonth->count(); i++)
        {
            dueDate* date = currentMonth->at(i);
            QDate paint = QDate::fromString(date->getDueDate(),Qt::ISODate);
            if (paint < QDate::currentDate())
            {   //we paint the background red if the item is over due
                brush.setColor(Qt::red);
            }
            else
            {
                QString table = *date->getTable();
                if (table == "pkeys")
                {   //we paint it green if it is a key that is due
                    brush.setColor(Qt::green);
                }
                else
                {   //we use mod 4 math to determine the colors of
                    //the other inventory items
                    int index = this->categories.indexOf(table);
                    index %= 4;
                    if (index == 0)
                    {
                        brush.setColor(Qt::cyan);
                    }
                    else if (index == 1)
                    {
                        brush.setColor(Qt::blue);
                    }
                    else if (index == 2)
                    {
                        brush.setColor(Qt::magenta);
                    }
                    else
                    {
                        brush.setColor(Qt::yellow);
                    }
                }
            }

            cf.setBackground(brush);
            ui->calendarWidget->setDateTextFormat(paint,cf);
        }
        //set the current year and month to what was passed in
        year = currYear;
        month = currMonth;
    }
}

//This pulls all items due on the selected day from the database,
//and displays a list to the user
void KeyWindow::getDueOnSelectedDay()
{
    for (int i = 0; i < ui->listWidgetCalendar->count(); i++)
    {
        delete ui->listWidgetCalendar->item(i);
    }
    ui->listWidgetCalendar->clear();

    QDate selectedDate = ui->calendarWidget->selectedDate();

    for (int i = 0; i < this->currentMonth->count(); i++)
    {
        dueDate* date = this->currentMonth->at(i);
        //This is only for keys because all other items have
        //The exact category name in the table field
        if (*date->getTable() == "pkeys")
        {
            date->setTable("Keys");
        }
        QDate itemise = QDate::fromString(date->getDueDate(),Qt::ISODate);
        if (itemise == selectedDate)
        {
            QListWidgetItem *item = new QListWidgetItem(*date->getTNumber()+
                                    "\t"+*date->getTable());
            QString num;
            num = num.setNum(date->getTransactionID());
            item->setToolTip(num);
            ui->listWidgetCalendar->addItem(item);
        }
    }
}

//This function displays a window with the information about
//The item checked out on that date and the person that has
//the item checked-out
void KeyWindow::displayDueDate(QListWidgetItem *item)
{
    inventory *dueItem;
    QString displayed = item->text();
    //Here we must split on tabs because that is how we displayed it
    QStringList data = displayed.split('\t');
    if (data.at(1) == "Keys")
    {
        dueItem = new key;
        dueItem->setTable("pkeys");
    }
    else
    {
        dueItem = new category_item;
        dueItem->setTable(data.at(1));
    }
    dueItem->setCheckOutTransaction(item->toolTip().toInt());

    person p;
    p.setTNumber(data.at(0));

    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->CalendarErrorLabel->setText("Database connection re-established.");

        }
        this->db->getPersonInfo(&p);
        this->db->getCheckOutInfo(dueItem,&p);
        this->db->getInventoryInfo(dueItem);

    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CalendarErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CalendarErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if ( i == 6)
        {
            //Category no longer exists
            setErrorLabel(ui->CalendarErrorLabel,"That item's category has been removed.");
        }
        else if (i == 200)
        {
            //Item not found
            setErrorLabel(ui->CalendarErrorLabel,"That person or check out or inventory could not be found.<br>It could have been removed by someone else.");
        }
        else
        {
            setErrorLabel(ui->CalendarErrorLabel,"There was an unknown error.");
        }
        return;
    }
    QString* show = new QString("--Person--\t--Inventory--\n");
    *show += "\n-Name-\t";
    if (*dueItem->getTable() == "pkeys")
    {//keys are special again
        key* k = (key*)dueItem;
        QString num;
        num = num.setNum(k->getTagNum());
        *show += "-Key Name-\n";
        *show += *p.getFirstName()+" "+*p.getLastName()+"\t"+*k->getKeyName()+"\n";
        *show += "\n-T-Number-\t-Tag Number-\n";
        *show += *p.getTNumber()+"\t"+num+"\n";
    }
    else
    {
        category_item* ci = (category_item*)dueItem;
        *show += "-Descriptor-\n";
        *show += *p.getFirstName()+" "+*p.getLastName()+"\t"+*ci->getDescriptor()+"\n";
        *show += "\n-T-Number-\n";
        *show += *p.getTNumber()+"\n";
    }
    *show += "\n-Number Out-\t-Date Out-\t-Date Due-\n";
    QString num;
    num = num.setNum(dueItem->getNumberCheckedOut());
    *show += num+"\t"+dueItem->getDateOut()+"\t"+dueItem->getDateDue()+"\n";

    *show += "\n Would you like to see this person in the system?\n";

    noRoom n(*show);
    if (n.exec() == QDialog::Rejected)
    {
        delete dueItem;
        delete show;
        return;
    }
    delete dueItem;
    delete show;
    ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);
    ui->lineEdit_Tnum->setText(*p.getTNumber());
    ui->listWidgetPeople->clearSelection();
    queryPeople();
}

//This function takes care of checking out an inventory item
//if a due date is requested from the user.
void KeyWindow::checkout_calendar()
{
    QDate dueDate = ui->dateEdit->date();
    QDate date = QDate::currentDate();
    if (dueDate < date){
        setErrorLabel(ui->CalendarErrorLabel,"You cannot select a due date in the past.");
        return;
    }
    qDebug() << dueDate.toString(Qt::ISODate);
    for (int i = 0; i < this->inv->count(); i++)
    {
        this->inv->at(i)->setDateDue(dueDate.toString(Qt::ISODate));
    }
    //qDebug() << this->inv->getDateDue();

    try{
        if (this->db == NULL){
           this->db = new Database("inventory");
           ui->CalendarErrorLabel->setText("Database connection re-established.");

        }
        for (int i = 0; i < this->inv->count(); i++)
        {
            //This will update the inventory item with
            //the available inventory, specifically.
            //this could change due to another user being on the system
            //at the same time
            inventory * item = this->inv->at(i);
            this->db->getInventoryInfo(item);
            if (item->getCurrentNumber() > item->getAmountAvailable())
            {
                setErrorLabel(ui->CalendarErrorLabel,"There is not enough of that inventory to checkout.<br>Please try again.");
                pathEnd();
                return;
            }
        }
        //all of items checked out will be of same category
        if (*this->inv->at(0)->getTable() == "pkeys")
        {//If checking-out keys the borrower needs to print
            //of and sign an agreement, for that we need the room lists
            QStringList roomsForAgreement;
            for (int i = 0; i < this->inv->count(); i++)
            {
                key* k = (key*)this->inv->at(i);
                QList<room*>* rooms = this->db->getKeyRooms(k);
                QString roomList = "";
                bool first = true;
                for (int j = 0; j < rooms->count(); j++)
                {
                    if (!first)
                    {
                        roomList += ", ";
                    }
                    first = false;
                    roomList += *rooms->at(j)->getRoomName();
                }
                roomsForAgreement.append(roomList);
                qDeleteAll(rooms->begin(),rooms->end());
                delete rooms;
            }

            Agreement a(this->per,this->inv,&roomsForAgreement);
            if (a.exec() != QDialog::Accepted)
            {
                pathEnd();
                return;
            }
        }
        db->transaction();
        for (int i = 0; i < this->inv->count(); i++)
        {
            if(!this->db->CheckOutInventory(this->inv->at(i),this->per))
            {
                setErrorLabel(ui->CalendarErrorLabel,"That person could not check out those items.");
                db->rollback();
                pathEnd();
                return;
            }
        }
        db->commit();
        ui->CalendarErrorLabel->setText("Those items were checked-out.");
        pathEnd();
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CalendarErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CalendarErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->CalendarErrorLabel,"That category no longer exists.<br>Please check on this.");
        }
        else if (i == 200)
        {
            setErrorLabel(ui->CalendarErrorLabel,"That person or item could not be found.<br>Please try again.");
        }
        else
        {
            setErrorLabel(ui->CalendarErrorLabel,"There was an unknown error.");
        }
    }
}

//Cancel the current check-out in progress
void KeyWindow::cancelPath_calendar()
{
    pathEnd();
    ui->CalendarErrorLabel->setText("You have successfully canceled.");
}

void KeyWindow::setCalendarDate()
{
    emit selectedDate(ui->dateEdit->date());
}

