#include <QtGui>
#include "keywindow.h"
#include "ui_keywindow.h"
#include "database.h"
#include "noroom.h"

#include "group.h"
#include "room.h"
#include "person.h"

extern void setErrorLabel(QLabel*, QString);
/////////////////////////////////////////////////
// Groups
//////////////////////////////////////////////////

//This connects buttons and other signals to functions in the
//system as well as setting limits on user input
void KeyWindow::groupConnections()
{
    ui->lineEdit_GroupName->setMaxLength(30);
    connect(ui->addGroup_but,SIGNAL(clicked()),this,SLOT(addGroup()));
    connect(ui->removeGroup_but,SIGNAL(clicked()),this,SLOT(deleteGroup()));
    connect(ui->listWidgetGroups,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(groupPrepare(QListWidgetItem*)));
    connect(ui->listWidgetGroups,SIGNAL(itemDoubleClicked(QListWidgetItem*)),this,SLOT(displayGroupInfo(QListWidgetItem*)));
    connect(ui->searchPeopleinGroup_but,SIGNAL(clicked()),this,SLOT(findPeopleInGroups()));
    connect(ui->addPeopletoGroup_but,SIGNAL(clicked()),this,SLOT(addPersonToGroup_group()));
    connect(ui->removeAllfromGroup_but,SIGNAL(clicked()),this,SLOT(removeAllPeople()));
    connect(ui->removeRoomsfromGroup_but,SIGNAL(clicked()),this,SLOT(modifyGroupRoomsList_remove()));
    connect(ui->addRoomstoGroup_but,SIGNAL(clicked()),this,SLOT(modifyGroupRoomsList_add()));
    connect(ui->GroupClear_but,SIGNAL(clicked()),this,SLOT(groupClear()));
}

//This displays the group name, list of rooms opened by the group,
//and list of people( names and TNumbers) when a groups list item
//is double clicked
void KeyWindow::displayGroupInfo(QListWidgetItem* item)
{
    //The group ID is "hidden" in the tool-tip for the list item
    group g = group(item->toolTip().toInt(),item->text());
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->GroupErrorLabel->setText("Database connection re-established.");
        }
        QList<room*>* rooms = this->db->getGroupRooms(&g);
        QList<person*>* people = this->db->getGroupPeople(&g);

        QString text = getGroupInfoString(&g,rooms,people);

        ui->textBrowserGroups->setText(text);
        qDeleteAll(rooms->begin(),rooms->end());
        qDeleteAll(people->begin(),people->end());
        delete rooms;
        delete people;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
        return;
    }

}

//Add a group to the system, it takes a Group name from the user
void KeyWindow::addGroup()
{
    //The group name must not be empty
    QString groupName = ui->lineEdit_GroupName->text();
    if (groupName.length() == 0)
    {
        setErrorLabel(ui->GroupErrorLabel,"A group name must be given to add a group.");
        return;
    }
    //we need a transaction because the user could be adding rooms
    this->db->transaction();
    group g;
    g.setGroupName(groupName);
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->GroupErrorLabel->setText("Database connection re-established.");
        }
        this->db->addGroup(&g);
        if (g.getGroupID() == 0)
        {
            setErrorLabel(ui->GroupErrorLabel,"That group could not be added successfully");
            this->db->rollback();
            return;
        }


        QString rooms = ui->lineEdit_GroupRoomNames->text();
        QStringList roomsList;
        QList<room*>* add = new QList<room*>;
        if (rooms.length() != 0)
        {
            //see keywindow.cpp
            roomsList = getTrimmedRooms(rooms);
            for (int i = 0; i < roomsList.count(); i++)
            {
                room *r = new room;
                r->setRoomName(roomsList.at(i));

                this->db->doesRoomExist(r);
                if (r->getRoomID() == 0)
                {
                    //we ask the user about the rooms, this pop-up was a user request
                    noRoom n("The room named: "+*r->getRoomName()+" was not found.\nWould you like to add it?");
                    if(n.exec() == QDialog::Rejected)
                    {
                        this->db->rollback();
                        setErrorLabel(ui->RoomErrorLabel,"That group could not be added with those rooms");
                        return;
                    }
                    this->db->addRoom(r);

                }
                add->append(r);
            }

            //in the database_groups.cpp
            this->db->modifyRoomList(Database::TO_GROUP,Database::ADD_ROOMS,g.getGroupID(),add);
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 4)
        {
            setErrorLabel(ui->GroupErrorLabel,"There is already a group with that name.<br>It could also be a room with a duplicate name, but it is unlikely.");
        }
        else if (i == 7)
        {
            setErrorLabel(ui->GroupErrorLabel,"Either that group, or one of those keys, no longer<br>exists in the system. Please check.");
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
        this->db->rollback();
        return;
    }


    if(this->db->commit())
    {
        listGroups();
        ui->GroupErrorLabel->setText("That group was successfully added");
    }
    else
    {
        setErrorLabel(ui->GroupErrorLabel,"That group could not be added to the system.");
    }

}

//This function tries to delete a group from the system, it takes
//a group name from the user
void KeyWindow::deleteGroup()
{
    //Group name must be non-empty
    QString groupName = ui->lineEdit_GroupName->text();
    if (groupName.length() == 0){
        setErrorLabel(ui->GroupErrorLabel,"A group name must be given to remove a group.");
        return;
    }
    group remove;
    remove.setGroupName(groupName);
    try
    {
        if (this->db == NULL)
        {
            this->db = new Database("inventory");
            ui->GroupErrorLabel->setText("Database connection re-established.");
        }
        if (this->db->removeGroup(&remove))
        {
            ui->GroupErrorLabel->setText("That group was removed from the system");
            listGroups();
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"That group could not be removed from the system.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
    }
}

//This clears the user input lines
void KeyWindow::groupClear()
{
    ui->lineEdit_GroupName->setText("");
    ui->lineEdit_GroupRoomNames->setText("");
}

//This gets all groups in the sysetm and displays them
//in the QListWidget on the groups tab
void KeyWindow::listGroups()
{
    try{
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->GroupErrorLabel->setText("Database connection re-established.");

        }
        QList<group*>* groups = this->db->getGroups();
        for (int i = 0; i < ui->listWidgetGroups->count(); i++)
        {//get rid of the old items
            delete ui->listWidgetGroups->item(i);
        }
        ui->listWidgetGroups->clear();
        if (groups->count() == 0)
        {
            qDeleteAll(groups->begin(),groups->end());
            delete groups;
            ui->GroupErrorLabel->setText("No groups are in the system.");
            groupClear();
            return;
        }
        for (int i = 0; i < groups->count(); i++)
        {
            group* display = groups->at(i);

            QListWidgetItem *item = new QListWidgetItem(*display->getGroupName(),ui->listWidgetGroups);
            QString groupID;
            groupID = groupID.setNum(display->getGroupID());
            item->setToolTip(groupID);//"hide" the group ID in the tooltip
            ui->listWidgetGroups->addItem(item);

        }
        qDeleteAll(groups->begin(),groups->end());
        delete groups;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
    }
}

//This function selectes a group to add people to
//and then makes the function calls to begin that process
//It takes a group name from the user
void KeyWindow::addPersonToGroup_group()
{
    //The group name must be non-empty
    QString groupName = ui->lineEdit_GroupName->text();
    if (groupName.length() == 0)
    {
        setErrorLabel(ui->GroupErrorLabel,"A group name must be given to add a person to a group.");
        return;
    }
    group addPerson;
    addPerson.setGroupName(groupName);
    try {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->GroupErrorLabel->setText("Database connection re-established.");

        }
        //make sure the group exists
        this->db->doesGroupExist(&addPerson);
        if (addPerson.getGroupID() == 0)
        {
            setErrorLabel(ui->GroupErrorLabel,"That group does not exist.");
            return;
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
        return;
    }
    if (!setPath(KeyWindow::ADD_TO_GROUP))
    {
        setErrorLabel(ui->GroupErrorLabel,QString("You are in the process of either checking out an item or adding a person to a group already.\n")+
                      QString("Please either cancel or complete that before adding a person to a group."));
        return;
    }

    //instance variable used to hold the group for this
    this->grp = new group;
    this->grp->setGroupID(addPerson.getGroupID());
    this->grp->setGroupName(*addPerson.getGroupName());
    ui->PersonErrorLabel->setText("Please select a person to add to \ngroup: "+*grp->getGroupName());
    //move over to the people tab to complete this
    ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);

}

//This function takes a group name from the user and attempts
//to remove all members from it
void KeyWindow::removeAllPeople()
{
    //group name must be non-empty
    QString groupName = ui->lineEdit_GroupName->text();
    if (groupName.size() == 0)
    {
        setErrorLabel(ui->GroupErrorLabel,"You must enter a group name to remove<br>all of it's people.");
        return;
    }
    try
    {
        if (this->db == NULL)
        {
            this->db = new Database("inventory");
            ui->GroupErrorLabel->setText("Database connection re-established.");
        }
        group g;
        g.setGroupName(groupName);
        this->db->doesGroupExist(&g);
        if (g.getGroupID() == 0)
        {
            setErrorLabel(ui->GroupErrorLabel,"You must select a group.");
            return;
        }
        if (this->db->removeAllPeopleFromGroup(&g))
        {
            ui->GroupErrorLabel->setText("That group now has no members.");
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"That group's members could not be removed.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
    }
}

//This places the group name of the selected item
//  into the line edit to be used as user input
void KeyWindow::groupPrepare(QListWidgetItem* item)
{
    ui->lineEdit_GroupName->setText(item->text());
}

//This takes a group name and a comma seperated list of
//room names and attempts to make a connection between the
//group and each room
void KeyWindow::modifyGroupRoomsList_add()
{
    QString groupName = ui->lineEdit_GroupName->text();
    QString rooms = ui->lineEdit_GroupRoomNames->text();
    //neither may be empty
    if (groupName == "")
    {
        setErrorLabel(ui->GroupErrorLabel,"You must select a room name.");
        return;
    }
    if (rooms == "")
    {
        setErrorLabel(ui->GroupErrorLabel,"You must give at least one room name<br>to try to add rooms.");
        return;
    }
    QList<room*> add;
    group g;
    g.setGroupName(groupName);
    try
    {
        this->db->transaction();
        this->db->doesGroupExist(&g);
        QStringList roomsList;

        if (g.getGroupID() == 0)
        {
            setErrorLabel(ui->GroupErrorLabel,"That group no longer exists.");
            return;
        }
        roomsList = getTrimmedRooms(rooms);
        for (int i = 0; i < roomsList.count(); i++)
        {
            room *r = new room;
            r->setRoomName(roomsList.at(i));

            this->db->doesRoomExist(r);
            if (r->getRoomID() == 0)
            {//ask the user about new rooms
                noRoom n("The room named: "+*r->getRoomName()+" was not found.\nWould you like to add it?");
                if(n.exec() == QDialog::Rejected){
                    this->db->rollback();
                    setErrorLabel(ui->GroupErrorLabel,"Those rooms could not be added.");
                    return;
                }
                this->db->addRoom(r);
            }
            add.append(r);
        }


        if (!this->db->modifyRoomList(Database::TO_GROUP,Database::ADD_ROOMS,g.getGroupID(),&add))
        {
            this->db->rollback();
            setErrorLabel(ui->GroupErrorLabel,"Those rooms could not be added to the group.<br>Canceling.");
            qDeleteAll(add.begin(),add.end());
            return;
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 4)
        {
            setErrorLabel(ui->GroupErrorLabel,"There is already a room with that name.");
        }
        else if (i == 7)
        {
            setErrorLabel(ui->GroupErrorLabel,"The group has been deleted.\nPlease try again.");
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
        this->db->rollback();
        qDeleteAll(add.begin(),add.end());
        return;
    }


    if(this->db->commit())
    {
        ui->GroupErrorLabel->setText("Those rooms were successfully added to the group.");
    }
    else
    {
        setErrorLabel(ui->GroupErrorLabel,"Those rooms couldn't be added to the group.");
    }
    qDeleteAll(add.begin(),add.end());
}

//This function takes a group name and comma seperated room list
//  and tries to dis-associate the rooms from the group
void KeyWindow::modifyGroupRoomsList_remove(){
    QString groupName = ui->lineEdit_GroupName->text();
    QString rooms = ui->lineEdit_GroupRoomNames->text();
    //neither can be empty
    if (groupName == "")
    {
        setErrorLabel(ui->GroupErrorLabel,"You must select a group name.");
        return;
    }
    if (rooms == "")
    {
        setErrorLabel(ui->GroupErrorLabel,"You must give at least one room name<br>to try to remove rooms.");
        return;
    }
    group g;
    g.setGroupName(groupName);
    QList<room*> add;
    try
    {
        this->db->transaction();
        this->db->doesGroupExist(&g);
        if (g.getGroupID() == 0)
        {
            setErrorLabel(ui->GroupErrorLabel,"That group no longer exists.");
            return;
        }
        QStringList roomsList;

        roomsList = getTrimmedRooms(rooms);
        for (int i = 0; i < roomsList.count(); i++)
        {
            room *r = new room;
            r->setRoomName(roomsList.at(i));
            //If the room doesn't exist don't bother trying to remove it.
            this->db->doesRoomExist(r);
            if (r->getRoomID() != 0)
            {
                add.append(r);
            }
        }


        if (!this->db->modifyRoomList(Database::TO_GROUP,Database::REMOVE_ROOMS,g.getGroupID(),&add))
        {
            this->db->rollback();
            setErrorLabel(ui->GroupErrorLabel,"Those rooms could not be removed from the group.<br>Canceling.");
            qDeleteAll(add.begin(),add.end());
            return;
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 7)
        {
            setErrorLabel(ui->GroupErrorLabel,"That group does not exist in the system.");
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
        qDeleteAll(add.begin(),add.end());
        this->db->rollback();
        return;
    }


    if(this->db->commit())
    {
        ui->GroupErrorLabel->setText("Those rooms were successfully removed from the group.");
    }
    else
    {
        setErrorLabel(ui->GroupErrorLabel,"Those rooms couldn't be removed from the group.");
    }
    qDeleteAll(add.begin(),add.end());
}

//This function takes a group name and attempts to show the people in the group
//in the people tab ListWidget
void KeyWindow::findPeopleInGroups(){
    try{
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");
        }
        group g;
        QString groupName = ui->lineEdit_GroupName->text();
        if (groupName.length() == 0)
        {
            setErrorLabel(ui->GroupErrorLabel,"You must select a group.");
            return;
        }
        g.setGroupName(groupName);
        this->db->doesGroupExist(&g);
        if (g.getGroupID() == 0)
        {
            setErrorLabel(ui->GroupErrorLabel,"That group does not exist.");
            return;
        }
        //Get all the people from the database
        QList<person*>* people = this->db->getGroupPeople(&g);
        ui->listWidgetPeople->clear();
        ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);
        if (people->count() == 0)
        {
            qDeleteAll(people->begin(),people->end());
            delete people;
            ui->PersonErrorLabel->setText("No people found with that query.");
            personClear();
            return;
        }
        //add them to the person list widget
        for (int i = 0; i < people->count(); i++)
        {
            person* display = people->at(i);

            QListWidgetItem *item = new QListWidgetItem((*display->getLastName())
                                    +", "+(*display->getFirstName()),ui->listWidgetPeople);
            //"hide" Tnumber in tooltip
            item->setToolTip(*display->getTNumber());
            ui->listWidgetPeople->addItem(item);

        }
        qDeleteAll(people->begin(),people->end());
        delete people;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->GroupErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->GroupErrorLabel,"There was an unknown error.");
        }
    }
}

//This function constructs the string to display to the user about the selected
//group.
QString KeyWindow::getGroupInfoString(group* g, QList<room*>* rooms,QList<person*>* people){
    QString info;
    info += *g->getGroupName()+"\n";

    int i = 0;
    QString roomString = "\n--Rooms--\n";
    for (; i < rooms->count(); i++){
        room* display = rooms->at(i);
        roomString += *display->getRoomName()+"\n";
    }
    if (i == 0){
        info += "\nNo rooms are opened by this group.\n";
    }
    else {
        info += roomString;
    }

    i = 0;
    QString personString = "\n--People--\n";
    personString += "\n-Name-\t-T-Number-\n";
    for (; i < people->count(); i++){
        person* display = people->at(i);
        personString += *display->getLastName()+", "+*display->getFirstName()+"\t";
        personString += *display->getTNumber()+"\n";
    }
    if (i == 0){
        info += "\nThere are no people in this group.\n";
    }
    else {
        info += personString;
    }
    return info;
}

//////////////////////////////////////////////////
// End Groups
//////////////////////////////////////////////////
