#include <QtGui>
#include "keywindow.h"
#include "ui_keywindow.h"
#include "database.h"
#include "checkout.h"

#include "category_item.h"
#include "person.h"

extern void setErrorLabel(QLabel*,QString);

//This function connects buttons and signals to functions and places
//  restrictions on user input
void KeyWindow::otherInvConnections()
{
    ui->lineEdit_NewCategory->setMaxLength(20);
    ui->lineEdit_ItemID->setMaxLength(50);
    ui->lineEdit_Quantity->setValidator(this->quantityVal);

    connect(ui->AddCategory_but,SIGNAL(clicked()),this,SLOT(addCategory()));
    connect(ui->RemoveCategory_but,SIGNAL(clicked()),this,SLOT(removeCategory()));
    connect(ui->AddNewItem_but,SIGNAL(clicked()),this,SLOT(addItem()));
    connect(ui->AlterItemInventory_but,SIGNAL(clicked()),this,SLOT(changeItemQuantity()));
    connect(ui->SearchItems_but,SIGNAL(clicked()),this,SLOT(queryItems()));
    connect(ui->CheckoutItem_but,SIGNAL(clicked()),this,SLOT(checkout_otherInv()));
    connect(ui->listWidgetCategories,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(itemPrepare(QListWidgetItem*)));
    connect(ui->listWidgetCategories,SIGNAL(itemDoubleClicked(QListWidgetItem*)),this,SLOT(displayInventoryInfo(QListWidgetItem*)));
    connect(ui->selectionListCategories,SIGNAL(currentIndexChanged(int)),this,SLOT(itemClear(int)));
    connect(ui->searchPeopleWithInventory_but,SIGNAL(clicked()),this,SLOT(findPeopleWithInventory()));
    connect(ui->refreshCat_but,SIGNAL(clicked()),this,SLOT(updateCategories()));
}

//This gets and displays information about the inventory item
//  whose list item is double clicked on the Other Inventory tab
void KeyWindow::displayInventoryInfo(QListWidgetItem* item)
{
    category_item ci;
    ci.setDescriptor(item->text());
    ci.setTable(item->toolTip());
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
        }
        this->db->getInventoryInfo(&ci);

        QString text = getOtherInvInfoString(&ci);

        ui->textBrowserCategories->setText(text);

    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That category has been removed.\nPlease try again.");
        }
        else if (i == 200)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That item is not in the system.");
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
        return;
    }
}

//This takes a string indicating the desired category name and attempts to create
//  a new category in the system
void KeyWindow::addCategory()
{
    QString catName = ui->lineEdit_NewCategory->text();
    if (catName == "")
    {
        setErrorLabel(ui->CategoryErrorLabel,"You must give a name to the new category.");
        return;
    }
    //The following is a hack, if the category name has capital letters in it,
    //Qt sometimes says that it exists when it does not
    catName = catName.toLower();
    try
    {
        if (this->db == NULL){
           this->db = new Database("inventory");
        }
        this->db->transaction();
        if (this->db->addCategory(catName))
        {
            ui->CategoryErrorLabel->setText("That category was successfully added.");
            this->db->commit();
            //This function is here to update the categories
            updateCategories();
        }
        else
        {
            this->db->rollback();
            setErrorLabel(ui->CategoryErrorLabel,"That category could not be added.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 3)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That category already exists.<br>Please choose a new name.");
        }
        else if (i == 250)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The selected table is a restricted word in mysql.\nPlease choose another.");
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
        this->db->rollback();
        return;
    }
}

//This takes a string indicating the category name and tries to remove that category
// from the system
void KeyWindow::removeCategory()
{
    int index = ui->selectionListCategories->currentIndex();
    //It cannot be "None"
    if (index == 0)
    {
        setErrorLabel(ui->CategoryErrorLabel,"You must select a category to remove.");
        return;
    }
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
        }
        this->db->transaction();
        if (this->db->removeCategory(ui->selectionListCategories->currentText()))
        {
            ui->CategoryErrorLabel->setText("That category was successfully removed.");
            this->db->commit();
            //update the categories after delete
            updateCategories();
        }
        else
        {
            this->db->rollback();
            setErrorLabel(ui->CategoryErrorLabel,"That category could not be removed.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That category does not exist in the database.");
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
        return;
        this->db->rollback();
    }
}

//This will take a string indicating the category to add the item to, as well as
//  a string descriptor for the item and an optional descripton and an optional
//  initial quantity
void KeyWindow::addItem()
{
    QString table = ui->selectionListCategories->currentText();
    //Cannot be on "None"
    if (ui->selectionListCategories->currentIndex() == 0)
    {
        setErrorLabel(ui->CategoryErrorLabel,"You must select a category to add the item to.");
        return;
    }
    QString descriptor = ui->lineEdit_ItemID->text();
    //Must not have an empty descriptor
    if (descriptor.length() == 0)
    {
        setErrorLabel(ui->CategoryErrorLabel,"Please give the item a descriptor.");
        return;
    }
    //description can be empty though
    QString description = ui->plainTextEdit_ItemDescription->toPlainText();
    QString quantityStr = ui->lineEdit_Quantity->text();
    //default of 1 for inventory
    int quantity = 1;
    //If an initial quantity is given, it must be positive
    if (quantityStr != "")
    {
        quantity = quantityStr.toInt();
        if (quantity < 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"If you wish to set an initial inventory<br>it must be positive.");
            return;
        }
    }
    //new item with given information
    category_item item;
    item.setTable(table);
    item.setDescriptor(descriptor);
    item.setDescription(description);
    item.setTotal(quantity);
    item.setCurrentNumber(quantity);
    try
    {
        if (this->db == NULL){
           this->db = new Database("inventory");
        }
        if (this->db->addInventoryItem(&item))
        {
            ui->CategoryErrorLabel->setText("That item was added.");
            updateCategories();
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"That item could not be added.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 4)
        {
            setErrorLabel(ui->CategoryErrorLabel,"An item with that descriptor already exists");
        }
        else if (i == 8)
        {//we will hit the trigger first and determine the trigger does not exist
            setErrorLabel(ui->CategoryErrorLabel,"That category is no longer in the system.");
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
        return;
    }
}

//This function will take a category name, an inventory descriptor and an amount
//  to alter the inventory by
void KeyWindow::changeItemQuantity()
{
    QString table = ui->selectionListCategories->currentText();
    //cannot be "None"
    if (ui->selectionListCategories->currentIndex() == 0)
    {
        setErrorLabel(ui->CategoryErrorLabel,"You must select a category to update the item's quantity.");
        return;
    }
    QString descriptor = ui->lineEdit_ItemID->text();
    //Cannot be empty
    if (descriptor.length() == 0)
    {
        setErrorLabel(ui->CategoryErrorLabel,"Please give the item a descriptor.");
        return;
    }
    QString quantityStr = ui->lineEdit_Quantity->text();
    int quantity = 0;
    //Cannot be empty
    if (quantityStr == "")
    {
        setErrorLabel(ui->CategoryErrorLabel,"You must give an amount to alter the inventory by.");
        return;
    }
    quantity = quantityStr.toInt();
    category_item item;
    item.setTable(table);
    item.setDescriptor(descriptor);
    item.setCurrentNumber(quantity);
    try
    {
        if (this->db == NULL){
           this->db = new Database("inventory");
        }
        if (this->db->updateItemQuantity(&item))
        {
            ui->CategoryErrorLabel->setText("That quantity was changed.");
            updateCategories();
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"That quantity could not be changed.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost.\nPlease try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That category was removed from the system.");
        }
        else if (i == 200)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That item could not be found.");
        }
        else if (i == 300)
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was not enough of that item available to remove,\nplease checkin some to remove more.");
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
        return;
    }
    if (item.getAmountAvailable() == 0)
    {
        //This will remove the item from the ListWidget if it is in the list
        for (int i = 0; i < ui->listWidgetCategories->count(); i++)
        {
            QListWidgetItem * entry = ui->listWidgetCategories->item(i);
            if (entry->text() == descriptor)
            {
                delete entry;
                break;
            }
        }
    }
}

//This clears the user input portions of the GUI
void KeyWindow::itemClear(int i)
{
    ui->lineEdit_NewCategory->clear();
    ui->lineEdit_Quantity->clear();
    ui->lineEdit_ItemID->clear();
    ui->plainTextEdit_ItemDescription->clear();

}

//This function will place information from the selected person
//  into the user input places for the other functions
void KeyWindow::itemPrepare(QListWidgetItem* item)
{
    int index = this->categories.lastIndexOf(item->toolTip())+1;
    ui->selectionListCategories->setCurrentIndex(index);
    ui->lineEdit_ItemID->setText(item->text());
}

//This function takes a string indicating the category name, an optional (partial)
//  descriptor, and an optional (partial) description.
//It displays items in the category that match the given options, or all items,
//  if no options have been given
void KeyWindow::queryItems()
{
    category_item ci;
    //cannot be "None"
    if (ui->selectionListCategories->currentIndex() == 0)
    {
        setErrorLabel(ui->CategoryErrorLabel,"You must select a category to search in.");
        return;
    }

    ci.setTable(ui->selectionListCategories->currentText());
    ci.setDescriptor(ui->lineEdit_ItemID->text());
    ci.setDescription(ui->plainTextEdit_ItemDescription->toPlainText());
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->CategoryErrorLabel->setText("Database connection re-established.");

        }
        QList<category_item*>* items = this->db->getItems(&ci);

        ui->listWidgetCategories->clear();
        if (items->count() == 0)
        {
            delete items;
            ui->CategoryErrorLabel->setText("No items found with that query.");
            itemClear(0);
            return;
        }
        for (int i = 0; i < items->count(); i++)
        {
            category_item* display = items->at(i);

            QListWidgetItem *item = new QListWidgetItem((*display->getDescriptor()),ui->listWidgetCategories);
            item->setToolTip(*display->getTable());
            ui->listWidgetCategories->addItem(item);

        }
        qDeleteAll(items->begin(),items->end());
        delete items;
        //The following function is slow, so I hide it among other database calls
        updateCategories();
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed. Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost. Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That category was removed from the system.");
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
    }
}

//This function allows for the selection of one or more inventory items
//  and begins the process of checking out the inventory
void KeyWindow::checkout_otherInv()
{
    QList<QListWidgetItem*> items = ui->listWidgetCategories->selectedItems();
    QList<inventory*>* invs = new QList<inventory*>;
    try
    {
        if (this->db == NULL){
           this->db = new Database("inventory");
        }
        //If items are selected in the List Widget, use them
        if (items.count() > 0)
        {
            for (int i = 0; i < items.count(); i++)
            {
                category_item* ci = new category_item;
                ci->setTable(items.at(i)->toolTip());
                ci->setDescriptor(items.at(i)->text());
                this->db->getInventoryInfo(ci);
                if (ci->getAmountAvailable() != 0)
                {
                    invs->append(ci);
                }
                else
                {
                    setErrorLabel(ui->KeysErrorLabel,"At least one of those item was not<br>included because of non-availibility.");
                }
            }
        }
        else
        {//otherwise, use the stuff given in the lineEdits
            QString table = ui->selectionListCategories->currentText();
            if (ui->selectionListCategories->currentIndex() == 0)
            {
                delete invs;
                setErrorLabel(ui->CategoryErrorLabel,"You must select a category to update the item's quantity.");
                return;
            }
            QString descriptor = ui->lineEdit_ItemID->text();
            if (descriptor.size() == 0)
            {
                delete invs;
                setErrorLabel(ui->CategoryErrorLabel,"You must give an item descriptor.");
                return;
            }
            category_item* item = new category_item;
            item->setTable(table);
            item->setDescriptor(descriptor);
            this->db->getInventoryInfo(item);
            if (item->getAmountAvailable() != 0)
            {
                invs->append(item);
            }
            else
            {
                setErrorLabel(ui->KeysErrorLabel,"At least one of those items was not<br>included because of non-availibility.");
            }
        }

    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That category has been removed from the system.");
        }
        else if (i == 200)
        {
            setErrorLabel(ui->CategoryErrorLabel,"One of those items could not be found.");
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
        qDeleteAll(invs->begin(),invs->end());
        delete invs;
        return;
    }

    //Try to start a new multi-tab process, this block should not happen
    //  because of precautions I have taken in keywindow.cpp
    if (!setPath(KeyWindow::CHECK_OUT))
    {
        qDeleteAll(invs->begin(),invs->end());
        delete invs;
        setErrorLabel(ui->CategoryErrorLabel,QString("You are in the process of either checking out an item or adding a person to a group already.\n")+
                      QString("Please either cancel or complete that before adding a person to a group."));
        return;
    }
    if (invs->count() == 0)
    {
        qDeleteAll(invs->begin(),invs->end());
        delete invs;
        setErrorLabel(ui->CategoryErrorLabel,"None of those inventory items had any available<br>to check-out.");
        pathEnd();
        return;
    }
    this->inv = invs;
    //display the checkout dialog to give the number of items to checkout
    //  and to request a due date
    Checkout ch(inv,this);
    if (ch.exec() == QDialog::Rejected)
    {
        this->pathEnd();
        setErrorLabel(ui->CategoryErrorLabel,"This check-out has been canceled.");
        return;
    }
    else
    {
        ui->PersonErrorLabel->setText("Please select a person to check-out the inventory.");
        ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);
    }
}

//This function builds the string that will be shown to the user
//  when an item is double clicked
QString KeyWindow::getOtherInvInfoString(category_item* ci)
{
    QString info;
    info += "--Category--\n"+*ci->getTable()+"\n";
    info += "\n--Descriptor--\n"+*ci->getDescriptor()+"\n";
    info += "\n--Description--\n"+*ci->getDescription()+"\n";
    info += "\n-Total Number-\t-Number Available-\n";
    QString num;
    num = num.setNum(ci->getTotal());
    info += num+"\t";
    num = num.setNum(ci->getAmountAvailable());
    info += num+"\n";
    return info;
}

//This function takes a category name and a descriptor and displays all people
//  who have that item checked out in the ListWidget on the people tab
void KeyWindow::findPeopleWithInventory()
{
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");
        }
        //Cannot be "None"
        if (ui->selectionListCategories->currentIndex() == 0)
        {
            setErrorLabel(ui->CategoryErrorLabel,"You must select a category.");
            return;
        }
        QString descriptor = ui->lineEdit_ItemID->text();
        //CAnnot be empty
        if (descriptor == "")
        {
            setErrorLabel(ui->CategoryErrorLabel,"You must select a item to search with.");
            return;
        }
        category_item ci;
        ci.setTable(ui->selectionListCategories->currentText());
        ci.setDescriptor(descriptor);
        //get all of the rest of the inventory information
        this->db->getInventoryInfo(&ci);
        QList<person*>* people = this->db->getPeopleWithInventoryCheckedOut(&ci);

        ui->listWidgetPeople->clear();
        ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);
        if (people->count() == 0)
        {
            delete people;
            ui->PersonErrorLabel->setText("No people found with that query.");
            personClear();
            return;
        }
        for (int i = 0; i < people->count(); i++)
        {
            person* display = people->at(i);

            QListWidgetItem *item = new QListWidgetItem((*display->getLastName())
                                    +", "+(*display->getFirstName()),ui->listWidgetPeople);
            item->setToolTip(*display->getTNumber());
            ui->listWidgetPeople->addItem(item);

        }
        qDeleteAll(people->begin(),people->end());
        delete people;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->CategoryErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That category was removed from the system");
        }
        else if (i == 200)
        {
            setErrorLabel(ui->CategoryErrorLabel,"That item could not be found.");
        }
        else
        {
            setErrorLabel(ui->CategoryErrorLabel,"There was an unknown error.");
        }
    }
}
