#include <QtGui>
#include "keywindow.h"
#include "ui_keywindow.h"
#include "database.h"
#include "groupselection.h"
#include "checkin.h"
#include "noroom.h"
#include "agreement.h"

#include "room.h"
#include "person.h"
#include "category_item.h"
#include "key.h"
#include "group.h"

extern void setErrorLabel(QLabel*, QString);
///////////////////////////////////////////////////////////////////////
// $People
///////////////////////////////////////////////////////////////////////

//This function connects all of the buttons and signals to functions as well
//  as setting limitations on the user input
void KeyWindow::peopleConnections()
{
    connect( ui->AddP_but, SIGNAL(clicked()), this, SLOT(addPerson() ) );
    connect( ui->listWidgetPeople, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(displayPersonInfo(QListWidgetItem*)));
    connect(ui->clearP_but, SIGNAL(clicked()), this, SLOT(personClear()));
    connect(ui->listWidgetPeople,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(personPrepare(QListWidgetItem*)));
    connect(ui->DelP_but,SIGNAL(clicked()),this,SLOT(deletePerson()));
    connect(ui->SearchP_but,SIGNAL(clicked()),this,SLOT(queryPeople()));
    connect(ui->continuePerson_but,SIGNAL(clicked()),this,SLOT(continue_path()));
    connect(ui->cancelPerson_but,SIGNAL(clicked()),this,SLOT(cancelPath_person()));
    connect(ui->RemovePfromG_but,SIGNAL(clicked()),this,SLOT(removePersonFromGroup()));
    connect(ui->Checkin_but,SIGNAL(clicked()),this,SLOT(checkinInventory()));
    connect(ui->updatePerson_but,SIGNAL(clicked()),this,SLOT(updatePerson()));

    //This section changes the search mode to custom if one of the input areas
    //  is edited
    connect(ui->lineEdit_Email,SIGNAL(editingFinished()),this,SLOT(resetCustom()));
    connect(ui->lineEdit_Fname,SIGNAL(editingFinished()),this,SLOT(resetCustom()));
    connect(ui->lineEdit_Lname,SIGNAL(editingFinished()),this,SLOT(resetCustom()));
    connect(ui->lineEdit_Tnum,SIGNAL(editingFinished()),this,SLOT(resetCustom()));
    connect(ui->lineEdit_HPhone,SIGNAL(editingFinished()),this,SLOT(resetCustom()));
    connect(ui->lineEdit_WPhone,SIGNAL(editingFinished()),this,SLOT(resetCustom()));
    connect(ui->plainTextEdit_StrAddr,SIGNAL(textChanged()),this,SLOT(resetCustom()));

    ui->lineEdit_Fname->setMaxLength(20);
    ui->lineEdit_Lname->setMaxLength(30);
    ui->lineEdit_Tnum->setMaxLength(9);
    ui->lineEdit_Tnum->setValidator(this->tnumVal);
    ui->lineEdit_CellPhone->setMaxLength(12);
    ui->lineEdit_CellPhone->setValidator(this->phoneVal);
    ui->lineEdit_HPhone->setMaxLength(12);
    ui->lineEdit_HPhone->setValidator(this->phoneVal);
    ui->lineEdit_WPhone->setMaxLength(12);
    ui->lineEdit_WPhone->setValidator(this->phoneVal);
    ui->lineEdit_Email->setMaxLength(50);

    ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);
}

//This function takes a required TNumber, first name, last name, and optional
//  street address, home phone, cell phone, work phone, and e-mail address
//  and attmpts to add a user to the sysetm with the given information
void KeyWindow::addPerson()
{

    QString TNum = ui->lineEdit_Tnum->text();
    //only check made because the validator ensures that only one that is
    //  9 charcters long can be valid
    if (TNum.size() != 9)
    {
        setErrorLabel(ui->PersonErrorLabel,"Invalid T-Number entered.");
        return;
    }

    //these cannot be empty
    QString FName = ui->lineEdit_Fname->text();
    if (FName.size() == 0)
    {
        setErrorLabel(ui->PersonErrorLabel,"The person must have a first name.");
        return;
    }
    QString LName = ui->lineEdit_Lname->text();
    if (LName.size() == 0)
    {
        setErrorLabel(ui->PersonErrorLabel,"The person must have a last name.");
        return;
    }


    person add;
    add.setTNumber(TNum);
    add.setFirstName(FName);
    add.setLastName(LName);



    QString HomeAddr = ui->plainTextEdit_StrAddr->toPlainText();
    if (HomeAddr.size() != 0)
    {
        add.setStreetAddress(HomeAddr);
    }

    QString HomePhone = ui->lineEdit_HPhone->text();
    //againg the validator ensures that if it is less then twelve characters long
    //  it cannot be valid, this applies to all phone numbers
    if (HomePhone.size() != 0)
    {
        if (HomePhone.size() != 12)
        {
            setErrorLabel(ui->PersonErrorLabel,"Please enter a valid Home Phone number.");
            return;
        }
        add.setHomePhone(HomePhone);
    }

    QString WorkPhone = ui->lineEdit_WPhone->text();
    if (WorkPhone.size() != 0)
    {
        if (WorkPhone.size() != 12)
        {
            setErrorLabel(ui->PersonErrorLabel,"Please enter a valid Office Phone number.");
            return;
        }
        add.setWorkPhone(WorkPhone);
    }

    QString CellPhone = ui->lineEdit_CellPhone->text();
    if (CellPhone.size() != 0)
    {
        if (CellPhone.size() != 12)
        {
            setErrorLabel(ui->PersonErrorLabel,"Please enter a valid Cell Phone number.");
            return;
        }
        add.setCellPhone(CellPhone);
    }
    //check for valid email address
    QString Email = ui->lineEdit_Email->text();
    if (Email.size() != 0)
    {
        QStringList parts = Email.split('@');
        if (parts.count() != 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The email address must be of the<br>form (string)@(string).");
            return;
        }
        add.setEmail(Email);
    }

    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");
        }

        bool verdict = false;
        verdict = this->db->addPerson(&add);
        if (verdict)
        {
            ui->PersonErrorLabel->setText("That person was added successfully");
            personClear();
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"That person could not be added.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 4)
        {
            setErrorLabel(ui->PersonErrorLabel,"There is already a person with that T-Number.");
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
        }
    }

}

//This function requires a TNumber and can take optional first name, last name,
//  street address, home phone, work phone, cell phone, and email address from
//  the user.
//It attempts to update the person object in the database to the values passed in
//  EXCEPT the TNumber, it may not be changed
void KeyWindow::updatePerson()
{
    person p;
    QString TNum = ui->lineEdit_Tnum->text();
    //the validator ensures valid TNumber iff it is 9 chars
    if (TNum.size() != 9)
    {
        setErrorLabel(ui->PersonErrorLabel,"Invalid T-Number entered.");
        return;
    }

    p.setTNumber(TNum);
    bool empty = true;
    QString FName = ui->lineEdit_Fname->text();
    if (FName.size() != 0)
    {
        p.setFirstName(FName);
        empty = false;
    }
    QString LName = ui->lineEdit_Lname->text();
    if (LName.size() != 0)
    {
        p.setLastName(LName);
        empty = false;
    }


    QString HomeAddr = ui->plainTextEdit_StrAddr->toPlainText();
    if (HomeAddr.size() != 0)
    {
        p.setStreetAddress(HomeAddr);
        empty = false;
    }

    QString HomePhone = ui->lineEdit_HPhone->text();
    //again the validator ensures a valid phone number iff it is 12 characters
    if (HomePhone.size() == 12)
    {
        p.setHomePhone(HomePhone);
        empty = false;
    }
    else if (HomePhone.size() != 0)
    {
        setErrorLabel(ui->PersonErrorLabel,"Invalid Home Phone number entered.");
        return;
    }


    QString WorkPhone = ui->lineEdit_WPhone->text();
    if (WorkPhone.size() == 12)
    {
        p.setWorkPhone(WorkPhone);
        empty = false;
    }
    else if (WorkPhone.size() != 0)
    {
        setErrorLabel(ui->PersonErrorLabel,"Invalid Work Phone number entered.");
        return;
    }

    QString CellPhone = ui->lineEdit_CellPhone->text();
    if (CellPhone.size() == 12)
    {
        p.setCellPhone(CellPhone);
        empty = false;
    }
    else if (CellPhone.size() != 0)
    {
        setErrorLabel(ui->PersonErrorLabel,"Invalid Cell Phone number entered.");
        return;
    }

    QString Email = ui->lineEdit_Email->text();
    if (Email.size() != 0)
    {
        QStringList parts = Email.split('@');
        if (parts.count() != 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The email address must be of the form<br>(string)@(string).");
            return;
        }
        p.setEmail(Email);
        empty = false;
    }

    if (empty)
    {
        setErrorLabel(ui->PersonErrorLabel,"You must provide at least one field to update.");
        return;
    }
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");
        }

        bool verdict = false;
        verdict = this->db->updatePerson(&p);
        if (verdict)
        {
            ui->PersonErrorLabel->setText("That person was updated successfully");
            personClear();
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"That person could not be updated.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 200)
        {
             setErrorLabel(ui->PersonErrorLabel,"That person could not be found to update.");
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
        }
    }

}

//This function takes a Tnumber and attempts to delete the person with that TNumber
//  from the database
void KeyWindow::deletePerson()
{
    QString tnum = ui->lineEdit_Tnum->text();
    if (tnum.length() != 9)
    {
        setErrorLabel(ui->PersonErrorLabel,"You must give a valid TNumber.");
        return;
    }
    try
    {
        person remove;
        remove.setTNumber(tnum);
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");

        }
        //we could be dealing with multiple db actions
        this->db->transaction();
        //we get the inventory the person has checked out
        QList<inventory*>* inv = db->getPersonInv(&remove,this->categories);
        db->getPersonInfo(&remove);
        //Does the user really want to delete a user with inventory checked in
        if (inv->count() != 0)
        {
            noRoom n("Note: This person has inventory checked-out.\nIf she is deleted then all inventory is checked-in.\nWould you still like to remove her?");
            if (n.exec() == QDialog::Rejected)
            {
                qDeleteAll(inv->begin(),inv->end());
                delete inv;
                return;
            }
            for (int i = 0; i < inv->count(); i++)
            {//if they do, set up each item and try to check them in
                inventory* item = inv->at(i);
                item->setCurrentNumber(item->getNumberCheckedOut());
                if(!this->db->CheckInInventory(item)){
                    db->rollback();
                    setErrorLabel(ui->PersonErrorLabel,"One of the items could not be checked in.\nCanceling.");
                    qDeleteAll(inv->begin(),inv->end());
                    delete inv;
                    return;
                }
            }
            //all done with inventory
            qDeleteAll(inv->begin(),inv->end());
            delete inv;
        }
        if(this->db->removePerson(&remove))
        {//Here we search the list Widget to try and delete the item
            //representing the deleted person
            for (int i = 0; i < ui->listWidgetPeople->count(); i++)
            {
                QListWidgetItem *a = ui->listWidgetPeople->item(i);
                if (a->toolTip() == tnum)
                {
                    delete a;
                }
            }
            db->commit();
            ui->PersonErrorLabel->setText("That person was deleted successfully.");

        }
        else
        {
            db->rollback();
            setErrorLabel(ui->PersonErrorLabel,"That person could not be deleted");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->PersonErrorLabel,"At least one of the categories<br>you have record of no longer exits.");
        }
        else if (i == 200)
        {
            setErrorLabel(ui->PersonErrorLabel,"No matching T-Number found.");
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
        }
        db->rollback();
    }
}

//This function takes optional (and possibly partial) TNumber, First name, last name
//  street address, Home phone, work phone, cell phone and email address. AT LEAST
//  ONE MUST BE GIVEN for the Custom search.
//It can also search for all people, all people in groups, all people with keys
//  and all people with other inventory checked-out
//It tries to find all people in the system who match ALL the given parameters and
//  display them in the ListWidget
void KeyWindow::queryPeople()
{

    person search;
    Database::PersonQueryMode mode = Database::ALL;

    //If custom, look at the user input slots
    if (ui->selectionListPeople->currentIndex() == 0)
    {
        mode = Database::CUSTOM;
        bool empty = true;

        QString TNum = ui->lineEdit_Tnum->text();
        if (TNum.size() != 0)
        {
            search.setTNumber(TNum);
            empty = false;
        }

        QString FName = ui->lineEdit_Fname->text();
        if (FName.size() != 0)
        {
            search.setFirstName(FName);
            empty = false;
        }

        QString LName = ui->lineEdit_Lname->text();
        if (LName.size() != 0)
        {
            search.setLastName(LName);
            empty = false;
        }

        QString HomeAddr = ui->plainTextEdit_StrAddr->toPlainText();
        if (HomeAddr.size() != 0)
        {
            search.setStreetAddress(HomeAddr);
            empty = false;
        }

        QString HomePhone = ui->lineEdit_HPhone->text();
        if (HomePhone.size() != 0)
        {
            search.setHomePhone(HomePhone);
            empty = false;
        }

        QString WorkPhone = ui->lineEdit_WPhone->text();
        if (WorkPhone.size() != 0){
            search.setWorkPhone(WorkPhone);
            empty = false;
        }

        QString Email = ui->lineEdit_Email->text();
        if (Email.size() != 0)
        {
            search.setEmail(Email);
            empty = false;
        }

        QString CellPhone = ui->lineEdit_CellPhone->text();
        if (CellPhone.size() != 0)
        {
            search.setCellPhone(CellPhone);
            empty = false;
        }

        if (empty)
        {
            setErrorLabel(ui->PersonErrorLabel,"A search can not be run with no queries.");
            return;
        }
    }

    //The other modes
    else if (ui->selectionListPeople->currentIndex() == 1)
    {
        mode = Database::ALL;
    }
    else if (ui->selectionListPeople->currentIndex() == 2)
    {
        mode = Database::IN_GROUPS;
    }
    else if (ui->selectionListPeople->currentIndex() == 3)
    {
        mode = Database::WITH_KEYS;
    }
    else if (ui->selectionListPeople->currentIndex() == 4)
    {
        mode = Database::WITH_OTHER_ITEMS;
    }
    else
    {
        //shouldn't happen
        setErrorLabel(ui->PersonErrorLabel,"Invalid Query mode selected.");
        return;
    }
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");

        }
        QList<person*>* people = this->db->getPeople(mode,&search,&this->categories);

        ui->listWidgetPeople->clear();
        if (people->count() == 0)
        {
            qDeleteAll(people->begin(),people->end());
            delete people;
            ui->PersonErrorLabel->setText("No people found with that query.");
            personClear();
            return;
        }
        for (int i = 0; i < people->count(); i++)
        {//add people
            person* display = people->at(i);

            QListWidgetItem *item = new QListWidgetItem((*display->getLastName())
                                    +", "+(*display->getFirstName()),ui->listWidgetPeople);
            //"hide" TNumber in tooltip
            item->setToolTip(*display->getTNumber());
            ui->listWidgetPeople->addItem(item);

        }
        qDeleteAll(people->begin(),people->end());
        delete people;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->PersonErrorLabel,"At least one of the categories<br>you have record of no longer exits.");
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
        }
    }
}

//This function places the TNumber of the selcted person into the lineEdit
//to be used as user input
void KeyWindow::personPrepare(QListWidgetItem* item)
{
    personClear();
    ui->lineEdit_Tnum->setText(item->toolTip());
}

//This function sets the selection mode to custom
void KeyWindow::resetCustom()
{
    ui->selectionListPeople->setCurrentIndex(0);
}

//This function empties the user input
void KeyWindow::personClear()
{
    ui->lineEdit_Fname->clear();
    ui->lineEdit_Lname->clear();
    ui->lineEdit_Tnum->clear();
    ui->plainTextEdit_StrAddr->clear();
    ui->lineEdit_HPhone->clear();
    ui->lineEdit_WPhone->clear();
    ui->lineEdit_CellPhone->clear();
    ui->lineEdit_Email->clear();
    //ui->textBrowserPeople->clear();
}

//This function collects the inventory of the selected person and
//  then opens up a Checkin dialog to continue.
void KeyWindow::checkinInventory()
{
    QList<QListWidgetItem*> items = ui->listWidgetPeople->selectedItems();
    QString tnum;
    person p;
    QList<inventory*>* chosen;
    bool temp = false;
    if (items.count() == 1)
    {
        tnum = items.at(0)->toolTip();
        if (tnum == *selPersonInfo->getTNumber())
        {//If the persons is the same as the selPerson
            //then we do not need to make a database call yet
            chosen = selPersonInv;
            if (chosen->count() == 0)
            {
                setErrorLabel(ui->PersonErrorLabel,"That person has no inventory checked out.");
                return;
            }
        }
        else
        {   //Now we do need to make one
            p.setTNumber(items.at(0)->toolTip());
            try
            {
                temp = true;
                chosen = this->db->getPersonInv(&p,this->categories);
                if (chosen->count() == 0)
                {
                    setErrorLabel(ui->PersonErrorLabel,"That person has no inventory checked out.");
                    return;
                }

            }
            catch (int i)
            {
                if (i == 1)
                {
                    setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
                    delete this->db;
                    this->db = NULL;
                }
                else if (i == 2)
                {
                    setErrorLabel(ui->PersonErrorLabel,"The database connection was lost.<br>Please try again later.");
                    delete this->db;
                    this->db = NULL;
                }
                else if (i == 6)
                {
                    setErrorLabel(ui->PersonErrorLabel,"At least one of the categories you<br>know of no longer exists.");
                }
                else if (i == 200)
                {
                    setErrorLabel(ui->PersonErrorLabel,"The person could not be found.");
                }
                else
                {
                    setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
                }
                return;
            }
        }
    }
    //if nothing is selected, use old selPersonInv
    else
    {
        if (this->selPersonInv->count() == 0)
        {
            setErrorLabel(ui->PersonErrorLabel,"That person has no inventory checked out.");
            return;
        }
        chosen = selPersonInv;
    }
    //we now certainly have a list of inventory in chosen

    Checkin ch(chosen);
    if (ch.exec() == QDialog::Accepted)
    {
        ui->PersonErrorLabel->setText("Those inventory item(s) were checked in.");
    }
    //if chosen is from a temporary person, go ahead and delete
    if (temp)
    {
        qDeleteAll(chosen->begin(),chosen->end());
        delete chosen;
    }
}

//This function takes the selected person info and selected person groups
//  and invokes a dialog to allow the user to remove a person from those groups
void KeyWindow::removePersonFromGroup()
{
    if (this->selPersonGroups->count() == 0)
    {
        setErrorLabel(ui->PersonErrorLabel,"That person is not a member of any groups.");
        return;
    }
    GroupSelection gs(this->selPersonInfo,this->selPersonGroups);
    //All errors are handled in the dialog, but success should be displayed here
    if (gs.exec() == QDialog::Accepted)
    {
        ui->PersonErrorLabel->setText("That person was removed from those groups.");
    }

}

//This determines where the contine/Finish button should go to based upon the
//  paths instance variable
void KeyWindow::continue_path()
{
    KeyWindow::PATH path = isPathActive();
    if (path == KeyWindow::CHECK_OUT)
    {
        checkout_person();
    }
    else if (path == KeyWindow::ADD_TO_GROUP)
    {
        addPersonToGroup_person();
    }
    else
    {
        setErrorLabel(ui->PersonErrorLabel,"This shouldn't happen.<br>Something wrong has occurred.");
    }
}

//This function takes care of adding people to a
//group, it takes either a TNumber or one or more
//"people" being selected in the ListWidget
void KeyWindow::addPersonToGroup_person()
{
    //get the items selected
    QList<QListWidgetItem*> items = ui->listWidgetPeople->selectedItems();
    QList<person*> people;
    //go ahead and get the Tnumber from the box, just in case
    QString TNum = ui->lineEdit_Tnum->text();
    if (items.count() > 0)
    {
        for (int i = 0; i < items.count(); i++)
        {
            person* p = new person;
            p->setTNumber(items.at(i)->toolTip());
            people.append(p);
        }
    }
    else
    {//If there are no items selected
        //the validator ensures a valid tnumber iff it is 9 characters
        if (TNum.size() < 9)
        {
            setErrorLabel(ui->PersonErrorLabel,"Invalid T-Number entered.");
            return;
        }
        person* p = new person;
        p->setTNumber(TNum);
        people.append(p);
    }

    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");

        }
        this->db->doesGroupExist(grp);
        if (grp->getGroupID() == 0)
        {//group was deleted since group selection
            qDeleteAll(people.begin(),people.end());
            setErrorLabel(ui->PersonErrorLabel,"That group no longer exists.<br>Please try again later.");
            pathEnd();
            return;
        }
        for (int i = 0; i < people.count(); i++)
        {
            if(!this->db->addPersonToGroup(grp,people.at(i)))
            {
                qDeleteAll(people.begin(),people.end());
                setErrorLabel(ui->PersonErrorLabel,"Those people could not be added to the group.");
                pathEnd();
                return;
            }
        }
        ui->PersonErrorLabel->setText("Those people were successfully added to the group.");

        pathEnd();
        qDeleteAll(people.begin(),people.end());
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 200)
        {
            setErrorLabel(ui->PersonErrorLabel,"That person could not be found, please try again.");
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
        }
        qDeleteAll(people.begin(),people.end());
    }
}

//This function cancels either a add people to group or check-out process
void KeyWindow::cancelPath_person()
{
    pathEnd();
    ui->PersonErrorLabel->setText("You have successfully canceled.");
}

//This function will take a TNumber indicating a person who will be checking-out
//  the inventory selected on either the keys or other inventory screen
//It will also either check-out the items, or it will route the user to the
//  calendar screen if a due date is requested
void KeyWindow::checkout_person()
{
    QString TNum = ui->lineEdit_Tnum->text();
    //the validator ensures a valid tnumber iff it is 9 characters
    if (TNum.size() != 9){
        setErrorLabel(ui->PersonErrorLabel,"Invalid T-Number entered.");
        return;
    }
    //this is an instance variable
    per = new person;
    per->setTNumber(TNum);

    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");

        }
        //get the rest of the person's information
        this->db->getPersonInfo(this->per);
        if (this->dueDateRequest)
        {//head to the calendar screen if due date requested
            ui->tabWidget->setCurrentIndex(KeyWindow::CALENDAR);
            ui->CalendarErrorLabel->setText("Please select a due date for the inventory.");
            return;
        }

        //If we are here then we are checking out the item(s) now
        for (int i = 0; i < this->inv->count(); i++)
        {
            //This will update the inventory item with
            //the available inventory, specifically.
            //this could change due to another user being on the system
            //at the same time
            inventory * item = this->inv->at(i);
            this->db->getInventoryInfo(item);
            if (item->getCurrentNumber() > item->getAmountAvailable())
            {
                setErrorLabel(ui->PersonErrorLabel,"There is not enough of that inventory to checkout.<br>Please try again.");
                pathEnd();
                return;
            }
        }
        //all inventory checked out at a time will be of the same category
        if (*this->inv->at(0)->getTable() == "pkeys")
        {//only show the agreement screen for keys
            QStringList roomsForAgreement;
            for (int i = 0; i < this->inv->count(); i++)
            {//need to get all of the rooms and parse them out
                key* k = (key*)this->inv->at(i);
                QList<room*>* rooms = this->db->getKeyRooms(k);
                QString roomList = "";
                bool first = true;
                for (int j = 0; j < rooms->count(); j++)
                {
                    if (!first)
                    {
                        roomList += ", ";
                    }
                    first = false;
                    roomList += *rooms->at(j)->getRoomName();
                }
                roomsForAgreement.append(roomList);
                qDeleteAll(rooms->begin(),rooms->end());
                delete rooms;
            }
            Agreement a(this->per,this->inv,&roomsForAgreement);
            if (a.exec() != QDialog::Accepted)
            {
                pathEnd();
                return;
            }
        }
        this->db->transaction();
        for (int i = 0; i < this->inv->count(); i++)
        {
            if(!this->db->CheckOutInventory(this->inv->at(i),this->per))
            {
                setErrorLabel(ui->PersonErrorLabel,"That person could not check out those items.");
                this->db->rollback();
                pathEnd();
                return;
            }
        }
        this->db->commit();
        ui->PersonErrorLabel->setText("Those items were checked-out.");
        pathEnd();
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->PersonErrorLabel,"At least one of the categories you know<br>of no longer exists.");
        }
        else if (i == 200)
        {
            setErrorLabel(ui->PersonErrorLabel,"That person, or one of those inventory items<br>could not be found.\nPlease try again.");
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
        }
    }
}

//This function gets and displays a string with a person's information
//  when that person's item has been double clicked
void KeyWindow::displayPersonInfo(QListWidgetItem* item)
{

    //ui->textBrowserPeople->clear();
    //the following will be remade here in a moment
    delete selPersonInfo;
    qDeleteAll(selPersonGroups->begin(),selPersonGroups->end());
    delete selPersonGroups;
    qDeleteAll(selPersonInv->begin(),selPersonInv->end());
    delete selPersonInv;
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");

        }
        selPersonInfo = new person;
        selPersonInfo->setTNumber(item->toolTip());
        this->db->getPersonInfo(selPersonInfo);
        selPersonGroups = this->db->getPersonGroups(selPersonInfo);
        selPersonInv = this->db->getPersonInv(selPersonInfo,this->categories);

        QString text = getPersonInfoString(selPersonInfo,selPersonGroups,selPersonInv);


        ui->textBrowserPeople->setText(text);
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->PersonErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 6)
        {
            setErrorLabel(ui->PersonErrorLabel,"At least one of the categories you know<br>about no longer exists.");
        }
        else if (i == 200)
        {
            setErrorLabel(ui->PersonErrorLabel,"That person could not be found in the database.");
        }
        else
        {
            setErrorLabel(ui->PersonErrorLabel,"There was an unknown error.");
        }
        ui->textBrowserPeople->setText("No Person Selected");
    }

}

//This function constructs the string that is displayed to the user when the
//  person's item is double clicked
QString KeyWindow::getPersonInfoString(person* p, QList<group*>* groups,QList<inventory*>* inventory_items)
{
    QString info;
    //basic info
    info += *p->getFirstName()+" "+*p->getLastName()+"\n";
    info += *p->getTNumber()+"\n\n";
    info += "--Address--\n"+*p->getStreetAddress()+"\n\n";
    info += "--Home Phone Number--\n"+*p->getHomePhone()+"\n\n";
    info += "--Work Phone Number--\n"+*p->getWorkPhone()+"\n\n";
    info += "--Cell Phone Number--\n"+*p->getCellPhone()+"\n\n";
    info += "--Email Address--\n"+*p->getEmail()+"\n";

    int i = 0;
    //group info
    QString groupString = "\n--Groups--\n";
    for (; i < groups->count(); i++)
    {
        group* display = groups->at(i);
        groupString += *display->getGroupName()+"\n";
    }
    if (i == 0)
    {
        info += "\nA Memeber of no groups.\n";
    }
    else
    {
        info += groupString;
    }

    i = 0;
    //This gets more interesting
    QString inv = "\n--Inventory--\n";
    QString type = "";//this keeps track of which group of inventory I am on currently
    inventory* items;
    QDate test(0000,00,00);//if date mathes this, then no due date
    QString dueDate;
    for (; i < inventory_items->count(); i++)
    {
        items = inventory_items->at(i);
        //again keys are special
        if (*items->getTable() == "pkeys")
        {
            if (items->getTable() != type)
            {//the keys header
                inv += "\n--Keys--\n";
                inv += "\n-Tag Number-\t-Number Out-\t-Due Date-\n";
                //set the type for the duration of keys
                type = *items->getTable();
            }
            if (items->getDateDue() == test.toString())
            {
                dueDate = "N/A";
            }
            else
            {
                dueDate = items->getDateDue();
            }
            QString num;
            num = num.setNum(items->getNumberCheckedOut());
            inv += *((key*)items)->getKeyName()+"\t"+num+"\t"+dueDate+"\n";

        }
        else
        {//all other
            if (*items->getTable() != type)
            {//again header for each
                inv += "\n--"+*items->getTable()+"--\n";
                inv += "\n-Descriptor-\t-Number Out-\t-Due Date-\n";
                //and again set the type for the duration
                type = *items->getTable();
            }
            if (items->getDateDue() == test.toString())
            {
                dueDate = "N/A";
            }
            else
            {
                dueDate = items->getDateDue();
            }
            QString num;
            num = num.setNum(items->getNumberCheckedOut());
            inv += *((category_item*)items)->getDescriptor()+"\t"+num+"\t"+dueDate+"\n";
        }
    }
    if (i == 0){
        info += "\nNo items checked out.\n";
    }
    else {
        info += inv;
    }

    return info;
}

///////////////////////////////////////////////////
// $End_People
///////////////////////////////////////////////////
