#include <QtGui>
#include "keywindow.h"
#include "ui_keywindow.h"
#include "database.h"

#include "room.h"
#include "person.h"
#include "key.h"
#include "group.h"

extern void setErrorLabel(QLabel*, QString);
///////////////////////////////////////////////////
// $Rooms
///////////////////////////////////////////////////

//This function connects the buttons and signals to functions as well
//  as placing limitations on user input
void KeyWindow::roomConnections(){
    ui->RoomIDL->hide();
    ui->lineEdit_Rooms->setMaxLength(30);
    connect(ui->addR_but,SIGNAL(clicked()),this,SLOT(addRoom()));
    connect(ui->delR_but,SIGNAL(clicked()),this,SLOT(deleteRoom()));
    connect(ui->listWidgetRooms,SIGNAL(itemClicked(QListWidgetItem*)),this,SLOT(roomPrepare(QListWidgetItem*)));
    connect(ui->listWidgetRooms,SIGNAL(itemDoubleClicked(QListWidgetItem*)),this,SLOT(displayRoomInfo(QListWidgetItem*)));
    connect(ui->searchPwAccess_but,SIGNAL(clicked()),this,SLOT(findPeopleWithRooms()));
}

//This taks a room name and attempts to add a room with that name to the database
void KeyWindow::addRoom()
{
    QString roomName = ui->lineEdit_Rooms->text();
    //Room name cannot be empty
    if (roomName.length() == 0)
    {
        setErrorLabel(ui->RoomErrorLabel,"A room name must be given to add a room");
        return;
    }
    room r;
    r.setRoomName(roomName);
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->RoomErrorLabel->setText("Database connection re-established.");
        }
        this->db->addRoom(&r);
        if (r.getRoomID() != 0)
        {
            ui->RoomErrorLabel->setText("That room was added successfully");
            listRooms();
        }
        else
        {
            setErrorLabel(ui->RoomErrorLabel,"That room could not be added.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 4)
        {
            setErrorLabel(ui->RoomErrorLabel,"There is already a room with that name.");
        }
        else
        {
            setErrorLabel(ui->RoomErrorLabel,"There was an unknown error.");
        }
    }
}

//This takes a room name and attempts to remove the room with that name from the
//  system
void KeyWindow::deleteRoom()
{
    QString roomName = ui->lineEdit_Rooms->text();
    //again cannot be empty
    if (roomName.length() == 0)
    {
        setErrorLabel(ui->RoomErrorLabel,"A room name must be given to remove a room");
        return;
    }
    room r;
    r.setRoomName(roomName);
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->RoomErrorLabel->setText("Database connection re-established.");

        }
        if (this->db->removeRoom(&r))
        {
            ui->RoomErrorLabel->setText("That room was removed successfully");
            listRooms();
        }
        else
        {
            setErrorLabel(ui->RoomErrorLabel,"That room could not be removed.");
        }
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 200)
        {
            setErrorLabel(ui->RoomErrorLabel,"That room does not exist to remove.");
        }
        else
        {
            setErrorLabel(ui->RoomErrorLabel,"There was an unknown error.");
        }
    }
}

//This takes the room name from the item which has been selected
//  and puts it in the lineEdit as user input
void KeyWindow::roomPrepare(QListWidgetItem *item)
{
    ui->lineEdit_Rooms->setText(item->text());
}

//This function lists all rooms in the system in the ListWidget on the
//  rooms tab
void KeyWindow::listRooms()
{
    try
    {
        if (this->db == NULL){
           this->db = new Database("inventory");
           ui->RoomErrorLabel->setText("Database connection re-established.");

        }

        QList<room*>* rooms = this->db->getRooms();

        ui->listWidgetRooms->clear();
        if (rooms->count() == 0)
        {
            delete rooms;
            ui->RoomErrorLabel->setText("No rooms are in the system.");
            roomClear();
            return;
        }
        for (int i = 0; i < rooms->count(); i++)
        {
            room* display = rooms->at(i);

            QListWidgetItem *item = new QListWidgetItem(*display->getRoomName(),ui->listWidgetRooms);\
            QString roomID;
            roomID = roomID.setNum(display->getRoomID());
            item->setToolTip(roomID);
            ui->listWidgetRooms->addItem(item);

        }
        qDeleteAll(rooms->begin(),rooms->end());
        delete rooms;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->RoomErrorLabel,"There was an unknown error.");
        }
    }
}

//This clears the user input on the screen
void KeyWindow::roomClear()
{
    ui->lineEdit_Rooms->setText("");
}

//This function takes a room name and input from a check-box and attempts to
//  display all people who have access to the room given through a key or group
void KeyWindow::findPeopleWithRooms()
{
    bool keys = false;
    bool groups = false;
    Database::RoomQueryMode mode;
    //checking the status of the ckeckbox
    if (ui->keyAccess_cb->checkState() == Qt::Checked)
    {
        keys = true;
        mode = Database::KEYS_ONLY;
    }
    if (ui->groupAccess_cb->checkState() == Qt::Checked)
    {
        groups = true;
        mode = Database::GROUPS_ONLY;
    }
    if (keys && groups)
    {
        mode = Database::BOTH;
    }
    //demorgain's law
    if (!(keys || groups))
    {
        setErrorLabel(ui->RoomErrorLabel,"Please select whether you want group or key access.");
        return;
    }

    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
           ui->PersonErrorLabel->setText("Database connection re-established.");
        }
        QString roomName = ui->lineEdit_Rooms->text();
        //must not be empty
        if (roomName.size() == 0)
        {
            setErrorLabel(ui->RoomErrorLabel,"Please select a room.");
            return;
        }
        room r;
        r.setRoomName(roomName);
        this->db->doesRoomExist(&r);
        //make sure the rooms is in the system
        if (r.getRoomID() == 0){
            setErrorLabel(ui->RoomErrorLabel,"That room does not exist.");
            return;
        }
        QList<person*>* people = this->db->getPeopleWithAccessToRooms(mode,&r);

        ui->listWidgetPeople->clear();
        ui->tabWidget->setCurrentIndex(KeyWindow::PEOPLE);
        if (people->count() == 0)
        {//No people
            delete people;
            ui->PersonErrorLabel->setText("No people found with that query.");
            personClear();
            return;
        }
        for (int i = 0; i < people->count(); i++)
        {//adding the people to the list
            person* display = people->at(i);

            QListWidgetItem *item = new QListWidgetItem((*display->getLastName())
                                    +", "+(*display->getFirstName()),ui->listWidgetPeople);
            item->setToolTip(*display->getTNumber());
            ui->listWidgetPeople->addItem(item);

        }
        qDeleteAll(people->begin(),people->end());
        delete people;

    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->RoomErrorLabel,"There was an unknown error.");
        }
    }
}

//This function retrieves the string with the information about the double clicked
//  room and displays it to the user
void KeyWindow::displayRoomInfo(QListWidgetItem *item)
{
    //Tag number in tool tip and Key name in text
    room r = room(item->toolTip().toInt(),item->text());
    try
    {
        if (this->db == NULL)
        {
           this->db = new Database("inventory");
        }
        QList<key*>* keys = this->db->getRoomKeys(&r);
        QList<group*>* groups = this->db->getRoomGroups(&r);

        QString text = getRoomInfoString(&r,groups,keys);

        ui->textBrowserRooms->setText(text);
        qDeleteAll(keys->begin(),keys->end());
        qDeleteAll(groups->begin(),groups->end());
        delete keys;
        delete groups;
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection failed.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else if (i == 2)
        {
            setErrorLabel(ui->RoomErrorLabel,"The database connection was lost.<br>Please try again later.");
            delete this->db;
            this->db = NULL;
        }
        else
        {
            setErrorLabel(ui->RoomErrorLabel,"There was an unknown error.");
        }
        ui->textBrowserRooms->setText("No Room Selected");
    }
}

//This function generates the string that is displayed to the user when a room
//  item is double clicked
QString KeyWindow::getRoomInfoString(room* r,QList<group *> *groups, QList<key *> *keys)
{
    QString info;
    info += *r->getRoomName()+"\n";

    int i = 0;
    QString groupString = "\n--Groups--\n";
    for (; i < groups->count(); i++)
    {
        group* display = groups->at(i);
        groupString += *display->getGroupName()+"\n";
    }
    if (i == 0)
    {
        info += "\nNo groups access this room.\n";
    }
    else
    {
        info += groupString;
    }

    i = 0;
    QString keyString = "\n--Keys--\n";
    for (; i < keys->count(); i++)
    {
        key* display = keys->at(i);
        keyString += *display->getKeyName()+"\n";
    }
    if (i == 0)
    {
        info += "\nNo keys unlock this room.\n";
    }
    else
    {
        info += keyString;
    }

    return info;
}

///////////////////////////////////////////////////
// $End_Rooms
///////////////////////////////////////////////////
