#include "login.h"
#include "ui_login.h"
#include "dataconfig.h"
#include "database.h"

extern void setErrorLabel(QLabel*, QString);
extern bool readConfiguration(QLabel*);

//This sets up the gui for users login into the system.
Login::Login(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Login)
{
    ui->setupUi(this);
    ui->lineEdit_loginUsername->setMaxLength(20);
    ui->lineEdit_loginPassword->setMaxLength(20);
    ui->lineEdit_loginPassword->setEchoMode(QLineEdit::Password);
    ui->errorLabel->setText("");
    connect(ui->loginConfig_but,SIGNAL(clicked()),this,SLOT(config()));
    readConfiguration(ui->errorLabel);
}

//This function reads in the user name and password given,
//hashes the password and attempts to verify that the given
//creditentials are a user
void Login::accept()
{

    ui->errorLabel->clear();
    Database* admins = NULL;
    try
    {
        Database admins("admins");
        QString user = ui->lineEdit_loginUsername->text();
        QString pass = ui->lineEdit_loginPassword->text();

        if( admins.login(user,Database::hashPassword(user,pass)))
        {
            QDialog::accept();
        }
        setErrorLabel(ui->errorLabel,"That username and password combination is invalid. Please try again.");
        ui->errorLabel->show();
    }
    catch (int i)
    {
        if (i == 1)
        {
            setErrorLabel(ui->errorLabel,"The database connection failed. Please try again later.");
            ui->errorLabel->show();
        }
        else if (i == 2)
        {
            setErrorLabel(ui->errorLabel,"The database connection was lost. Please try again later.");
            ui->errorLabel->show();
            delete admins;
        }
        else
        {
            setErrorLabel(ui->errorLabel,"There was an unknown error.");
            ui->errorLabel->show();
        }
    }
    delete admins;
}

//This function opens a database configuration screen and then reads the new
//configuration if it was changed
void Login::config(){
    DataConfig d;
    if (d.exec() == QDialog::Accepted){
        readConfiguration(ui->errorLabel);
        ui->errorLabel->setText("Your database configuration was successfully saved.");
    }
}

Login::~Login()
{
    delete ui;
}
