#include "personadd.h"
#include "keywindow.h"
#include "database.h"
#include "ui_personadd.h"

extern void setErrorLabel(QLabel*, QString);

PersonAdd::PersonAdd(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::PersonAdd)
{
    ui->setupUi(this);
    try{
        this->db = new Database("addition");
    }
    catch (int i){
        qDebug() << "Error connecting" << endl;
    }
}

void PersonAdd::accept(){
    QString TNum = ui->lineEdit_PerAddTNum->text();
    if (TNum.size() != 9){
        setErrorLabel(ui->errorLabel,"Invalid T-Number entered.");
        return;
    }
    if(TNum.at(0) != 'T'){
        setErrorLabel(ui->errorLabel,"Invalid T-Number entered.");
        return;
    }
    for (int i = 1; i < TNum.length(); i++){
        if (TNum.at(i) > '9' || TNum.at(i) < '0'){
            setErrorLabel(ui->errorLabel,"Invalid T-Number entered.");
            return;
        }
    }
    QString FName = ui->lineEdit_PerAddFName->text();
    if (FName.size() == 0){
        setErrorLabel(ui->errorLabel,"The person must have a first name.");
        return;
    }
    QString LName = ui->lineEdit_PerAddLName->text();
    if (LName.size() == 0){
        setErrorLabel(ui->errorLabel,"The person must have a last name.");
        return;
    }

    QStringList params;

    QString HomeAddr = ui->plainTextEdit_PerAddHomeAdd->toPlainText();
    if (HomeAddr.size() != 0){
        params.append(HomeAddr);
    }
    else{
        params.append(NULL);
    }

    QString HomePhone = ui->lineEdit_PerAddHPhone->text();
    if (HomePhone.size() != 0){
        QStringList nums = HomePhone.split('-');
        if(nums.count() != 3){
            setErrorLabel(ui->errorLabel,"The home phone number must be of the form ###-###-###.");
            return;
        }
        for( int i = 0; i < nums.count(); i++){
            QString num = nums.at(i);
            int a = num.toInt();
            if (a == 0){
                setErrorLabel(ui->errorLabel,"The home phone number must be of the form ###-###-###.");
                return;
            }
        }
        params.append(HomePhone);
    }
    else{
        params.append(NULL);
    }

    QString WorkPhone = ui->lineEdit_PerAddOPhone->text();
    if (WorkPhone.size() != 0){
        QStringList nums = WorkPhone.split('-');
        if(nums.count() != 3){
            setErrorLabel(ui->errorLabel,"The home phone number must be of the form ###-###-###.");
            return;
        }
        for( int i = 0; i < nums.count(); i++){
            QString num = nums.at(i);
            int a = num.toInt();
            if (a == 0){
                setErrorLabel(ui->errorLabel,"The home phone number must be of the form ###-###-###.");
                return;
            }
        }
        params.append(WorkPhone);
    }
    else{
        params.append(NULL);
    }

    QString Email = ui->lineEdit_PerAddEmail->text();
    if (Email.size() != 0){
        QStringList parts = Email.split('@');
        if (parts.count() != 2){
            setErrorLabel(ui->errorLabel,"The email address must be of the form <string>@<string>.");
            return;
        }
        params.append(Email);
    }
    else {
        params.append(NULL);    }
    try{

        bool verdict = false;
        verdict = this->db->addPerson(TNum,FName,LName,params.at(0),params.at(1),params.at(2),params.at(3));
        if (verdict){
            //people error label = success
        }
        else {
            setErrorLabel(ui->errorLabel,"That person could not be added.");
        }
    }
    catch (int i){
        setErrorLabel(ui->errorLabel,"There was a database connection error, please try again later.");
    }


}

PersonAdd::~PersonAdd()
{
    delete db;
    delete ui;
}
