#include "user.h"
#include "ui_user.h"
#include <QCryptographicHash>


extern void setErrorLabel(QLabel*, QString);

//This constructor sets up the the gui for the user management screen
//attempts to create a database connection, and list the users.
User::User(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::User)
{

    ui->setupUi(this);
    ui->lineEdit_UserUsername->setMaxLength(20);
    ui->lineEdit_UserPassword->setMaxLength(20);
    ui->lineEdit_UserPassword->setEchoMode(QLineEdit::Password);
    ui->errLabel->clear();
    connect(ui->UserAddUser_but,SIGNAL(clicked()),this,SLOT(addUser()));
    connect(ui->UserDeleteUser_but,SIGNAL(clicked()),this,SLOT(deleteUser()));
    connect(ui->UserResetPass_but,SIGNAL(clicked()),this,SLOT(resetPassword()));
    connect(ui->listWidgetUsers,SIGNAL(itemDoubleClicked(QListWidgetItem*)),this,SLOT(userPrepare(QListWidgetItem*)));

    try{
        dbusers = new Database("admins");
    }
    catch (int i){
        if (i == 1){
            setErrorLabel(ui->errLabel,"The database connection failed. Please try again later.");
            ui->errLabel->show();
            delete this->dbusers;
            this->dbusers = NULL;
        }
    }
    listUsers();
}

//This function pulls a list of all users from from the database and then
//displays them to the user
void User::listUsers(){
    QStringList vec;
    try{
        if (this->dbusers == NULL){
            this->dbusers = new Database("admins");
        }
        vec = dbusers->getUsers();
    }
    catch (int i){
        if (i == 1){
            setErrorLabel(ui->errLabel,"The database connection failed. Please try again later.");
            ui->errLabel->show();
            delete this->dbusers;
            this->dbusers = NULL;
        }
        else if (i == 2){
            setErrorLabel(ui->errLabel,"The database connection was lost. Please try again later.");
            ui->errLabel->show();
            delete this->dbusers;
            this->dbusers = NULL;
        }
        else {
            setErrorLabel(ui->errLabel,"There was an unknown error.");
            ui->errLabel->show();
        }
    }

    ui->listWidgetUsers->clear();
    for (int i = 0; i < vec.size(); i++){
        ui->listWidgetUsers->addItem(vec[i]);
    }
}

//This function pulls username and password information from the screen,
//checks it for size, hashes the password and attempts to add the user to the system.
//any errors are displayed to the user
void User::addUser(){
    ui->errLabel->clear();
    QString pass = ui->lineEdit_UserPassword->text();
    QString user = ui->lineEdit_UserUsername->text();
    if (pass == "" || user == "" ){
        setErrorLabel(ui->errLabel,"Blank username or password not allowed");
        return;
    }
    if (pass.length() < 5)
    {
        setErrorLabel(ui->errLabel,"The given password is too short.");
        return;
    }
    if (user.length() < 3)
    {
        setErrorLabel(ui->errLabel,"The given password is too short.");
        return;
    }

    try{
        if (this->dbusers == NULL){
            this->dbusers = new Database("admins");
        }
        QByteArray passHash = Database::hashPassword(user,pass);
        if(!dbusers->addUser(user,passHash)){
            setErrorLabel(ui->errLabel,"That user could not be added.");
        }
        ui->errLabel->setText("User Added successfully.");
    }
    catch (int i){
        if (i == 1){
            setErrorLabel(ui->errLabel,"The database connection failed. Please try again later.");
            ui->errLabel->show();
            this->dbusers = NULL;
        }
        else if (i == 2){
            setErrorLabel(ui->errLabel,"The database connection was lost. Please try again later.");
            ui->errLabel->show();
            delete this->dbusers;
            this->dbusers = NULL;
        }
        else {
            setErrorLabel(ui->errLabel,"There was an unknown error.");
            ui->errLabel->show();
        }
    }
    listUsers();

}

//this function takes the user name from the currently selected user and
//attempts to remove that user from the system
void User::deleteUser(){
    if (ui->listWidgetUsers->currentRow() == -1){
        setErrorLabel(ui->errLabel,"No user selected to delete.");
        return;
    }
    QListWidgetItem *selection = ui->listWidgetUsers->item(ui->listWidgetUsers->currentRow());
    QString user = selection->text();
    try {
        if (this->dbusers == NULL){
            this->dbusers = new Database("admins");
        }
        if( !dbusers->removeUser(user)){
            setErrorLabel(ui->errLabel,"That user could not be deleted.");
            return;
        }
    }
    catch (int i){
        if (i == 1){
            setErrorLabel(ui->errLabel,"The database connection failed. Please try again later.");
            ui->errLabel->show();
            delete this->dbusers;
            this->dbusers = NULL;
        }
        else if (i == 2){
            setErrorLabel(ui->errLabel,"The database connection was lost. Please try again later.");
            ui->errLabel->show();
            this->dbusers = NULL;
        }
        else {
            setErrorLabel(ui->errLabel,"There was an unknown error.");
            ui->errLabel->show();
        }
    }
    ui->errLabel->setText("User successfully deleted.");
    qDeleteAll(ui->listWidgetUsers->selectedItems());
}

//This function takes a username and a password and after checking them for length,
//it hashes the password and attempts to update the password in the database for
//that user
void User::resetPassword(){
    ui->errLabel->clear();
    QString pass = ui->lineEdit_UserPassword->text();
    QString user = ui->lineEdit_UserUsername->text();
    if (pass == "" || user == "" ){
        setErrorLabel(ui->errLabel,"Blank username or password not allowed");
        return;
    }
    if (pass.length() < 5)
    {
        setErrorLabel(ui->errLabel,"The given password is too short.");
        return;
    }
    if (user.length() < 3)
    {
        setErrorLabel(ui->errLabel,"The given password is too short.");
        return;
    }

    try{
        if (this->dbusers == NULL){
            this->dbusers = new Database("admins");
        }
        QByteArray passHash = Database::hashPassword(user,pass);
        if(!dbusers->resetPass(user,passHash)){
            setErrorLabel(ui->errLabel,"The password could not be changed.");
        }
        ui->errLabel->setText("Password changed successfully.");
    }
    catch (int i){
        if (i == 1){
            setErrorLabel(ui->errLabel,"The database connection failed. Please try again later.");
            ui->errLabel->show();
            delete this->dbusers;
            this->dbusers = NULL;
        }
        else if (i == 2){
            setErrorLabel(ui->errLabel,"The database connection was lost. Please try again later.");
            ui->errLabel->show();
            delete this->dbusers;
            this->dbusers = NULL;
        }
        else {
            setErrorLabel(ui->errLabel,"There was an unknown error.");
            ui->errLabel->show();
        }
    }
}

//This fucntion places the username of the selected user in the
//lineEdit for the use of other functions
void User::userPrepare(QListWidgetItem *user){
    ui->lineEdit_UserUsername->setText(user->text());
    ui->lineEdit_UserPassword->clear();
}

User::~User()
{
    delete dbusers;
    delete ui;
}
