#include "dataconfig.h"
#include "ui_dataconfig.h"

#include <QFile>
#include <QTextStream>

extern void setErrorLabel(QLabel*, QString);

DataConfig::DataConfig(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DataConfig)
{
    extern QString dbhost;
    extern QString dbuser;
    extern QString dbpass;

    ui->setupUi(this);
    ui->lineEdit_DataConfigPass->setEchoMode(QLineEdit::Password);

    ui->lineEdit_DataConfigHost->setText(dbhost);
    ui->lineEdit_DataConfigUser->setText(dbuser);
    ui->lineEdit_DataConfigPass->setText(dbpass);

}

void DataConfig::accept(){
    extern QString dbhost;
    extern QString dbuser;
    extern QString dbpass;

    QString host = ui->lineEdit_DataConfigHost->text();
    QString user = ui->lineEdit_DataConfigUser->text();
    QString pass = ui->lineEdit_DataConfigPass->text();

    bool changed = false;

    if (host.length() == 0){
        setErrorLabel(ui->DataConfigErrorLabel,"You must enter a hostname.");
        return;
    }
    if (host != dbhost){
        changed = true;
    }

    if (user.length() == 0){
        setErrorLabel(ui->DataConfigErrorLabel,"You must enter a username.");
        return;
    }
    if (user != dbuser){
        changed = true;
    }

    if (pass != dbpass){
        changed = true;
    }

    if (changed){
        QFile oldF("KIA.conf");
        QFile newF("KIA.conf.tmp");

        if (!oldF.open(QFile::ReadOnly|QFile::Text) || !newF.open(QFile::WriteOnly|QFile::Text)){
            setErrorLabel(ui->DataConfigErrorLabel,"The file could not be opened. Please try again later.");
            return;
        }

        QTextStream in(&oldF);
        QTextStream out(&newF);
        while(!in.atEnd()){
            QString line = in.readLine();
            QStringList sp = line.split(':');
            if (sp.at(0) == "hname"){
                line = "hname:"+host;
            }
            else if (sp.at(0) == "MySQLu"){
                line = "MySQLu:"+user;
            }
            else if (sp.at(0) == "MySQLp"){
                line = "MySQLp:"+pass;
            }
            out << line << endl;
        }

        oldF.close();
        newF.close();

        if (!oldF.remove()){
            setErrorLabel(ui->DataConfigErrorLabel,"Error, could not remove old config file.");
            return;
        }
        if (!newF.rename("KIA.conf")){
            setErrorLabel(ui->DataConfigErrorLabel,"Error, could not rename new config file.");
            return;
        }
        QDialog::accept();
    }
    else {
        QDialog::reject();
    }
}

DataConfig::~DataConfig()
{
    delete ui;
}
