#include "setup.h"
#include "ui_setup.h"
#include "database.h"
#include "dataconfig.h"

QString dbuser;
QString dbpass;
QString dbhost;

extern void setErrorLabel(QLabel*, QString);

setup::setup(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::setup)
{
    ui->setupUi(this);
    readConfiguration(ui->label);

    dbusers = NULL;

    connect(ui->pushButton_db,SIGNAL(clicked()),this,SLOT(config()));
}

setup::~setup()
{
    delete dbusers;
    delete ui;
}

void setup::accept()
{
    QString pass = ui->lineEdit_password->text();
    QString user = ui->lineEdit_username->text();
    if (pass == "" || user == "" ){
        setErrorLabel(ui->label,"Blank username or password not allowed");
        return;
    }

    try{
        if (this->dbusers == NULL){
            this->dbusers = new Database("admins");
        }
        dbusers->setupTables();
        QByteArray passHash = Database::hashPassword(user,pass);
        if(!dbusers->setInitialUser(user,passHash)){
            setErrorLabel(ui->label,"That user could not be added.");
        }
        ui->label->setText("The system has been set up and the user has been added.\nPlease close this and log into KIA.");
    }
    catch (int i){
        if (i == 1){
            setErrorLabel(ui->label,"The database connection failed. Please try again later.");
            ui->label->show();
            this->dbusers = NULL;
        }
        else if (i == 2){
            setErrorLabel(ui->label,"The database connection was lost. Please try again later.");
            ui->label->show();
            delete this->dbusers;
            this->dbusers = NULL;
        }
        else if (i == 4){
            setErrorLabel(ui->label,"There is already a user with that username, please choose another.");
            ui->label->show();
        }
        else {
            setErrorLabel(ui->label,"There was an unknown error.");
            ui->label->show();
        }
    }

}

void setup::config(){
    DataConfig d;
    if (d.exec() == QDialog::Accepted){
        readConfiguration(ui->label);
        ui->label->setText("Your database configuration was successfully saved.");
    }
}

bool setup::readConfiguration(QLabel *errorLabel){
    QFile inF("KIA.conf");
    if (!inF.open(QFile::ReadOnly|QFile::Text)){
        if (errorLabel != NULL){
            setErrorLabel(errorLabel,"The configuration file could not be read.");
            return false;
        }
    }
    QTextStream in(&inF);
    while (!in.atEnd()){
        QStringList sp = in.readLine().split(':');
        if (sp.at(0) == "hname"){
            dbhost = sp.at(1);
        }
        else if (sp.at(0) == "MySQLu"){
            dbuser = sp.at(1);
        }
        else if (sp.at(0) == "MySQLp"){
            if (sp.count() == 2){
                dbpass = sp.at(1);
            }
            else {
                dbpass = "";
            }
        }
    }

   inF.close();
   return true;
}
